//---------------------------------------------------------------------------
// AVI format : studied by myself
// REAL, OGG format : big help to start understanding the format from
// Quovadis (quovadis@nbsi.de) the author of NBS Catalogue (http://nbsi.de).
//---------------------------------------------------------------------------
#include <vcl.h>						// ShowMesssage
#pragma hdrstop
#pragma warn -pin						// Suppr warning "Init is partially bracketed"

#include <dshow.h>						// DirectShow
#include <dvdmedia.h>					// VIDEOINFOHEADER2
#include <io.h>							// filelength
#include <stdio.h>						// fopen
#include <vfw.h>						// AVIFileInit
#include "qedit.h"						// IMediaDet

#include "Main.h"						// MainForm
#include "fileinfo.h"
#include "multinfo.h"
#include "mpeginfo.h"

//#define DEBUG	1

extern struct FileInfo *pai;

char	szErr[100];

// Open an AVI file and retrieve main information
// -----------------------------------------------------------

//#define USEAVIFUNC 1

int GetAVIinfo (LPCSTR szFile)
{
	int		nbs,
			si2wfx,
			as;
	LONG	size;
	AVISTREAMINFO	si[MAXSTREAMS],		// Streams information
				*pasi;
	WAVEFORMATEX	wfx[MAXSTREAMS];	// Wave information

#ifdef USEAVIFUNC
	LONG		hr;
	PAVIFILE 	pfile;
	PAVISTREAM	avis;					// Stream-interface
#else
	FILE	*fh;
	char	szBuf[MAXHEAD+1];
	int		len,
            pos,
            totpos;
	DWORD	*pdw;
#endif

    for (int i=0 ; i<MAXSTREAMS ; i++)
    	memset (&si[i],  0, sizeof (AVISTREAMINFO));
    for (int i=0 ; i<MAXAUDIOSTREAMS ; i++)
    	memset (&wfx[i],  0, sizeof (WAVEFORMATEX));

#ifdef USEAVIFUNC
	hr = AVIFileOpen (&pfile, szFile, OF_SHARE_DENY_WRITE, 0L);
	if (hr)
	{	strcpy (szErr, szErrorString[5]); return -1;	}

	// Open the streams until a stream is not available.

	for (nbs=0 ; nbs < MAXSTREAMS ; nbs++) {
		avis = NULL;
		if (AVIFileGetStream (pfile, &avis, 0L, nbs) != AVIERR_OK)
			break;
		if (avis == NULL)
			break;
		AVIStreamInfo (avis, si[nbs], sizeof(AVISTREAMINFO));
		if (si[nbs].fccType == streamtypeAUDIO)
		{
			size = sizeof(WAVEFORMATEX);
			as = pai->bAStreamsNb++;
			AVIStreamReadFormat (avis, 0, &wfx[as], &size);
            si2wfx = nbs - as;
		}
	}
	AVIFileRelease (pfile);  // closes the file
	if (!nbs)
	{	strcpy (szErr, szErrorString[6]); return -2;	}
#else
	// Header reading

	fh = fopen (szFile, "rb");
	if (fh == NULL)
	{	strcpy (szErr, szErrorString[7]);	return -1;	}

	len = fread (szBuf, 1, MAXHEAD, fh);
	if (!len || (len < 512))
	{
    	fclose (fh);
    	strcpy (szErr, szErrorString[8]);
        return -2;
    }

	// Header control

	pdw = (DWORD *)szBuf;
	pdw++;
	pai->dwFileTheoSize = *pdw + 8;
	if ((pai->dwFileTheoSize > pai->dwFileRealSize) && !strstr (szFile, "hdr") && !strstr (szFile, "head"))
	{
		char	szMess[100];
		wsprintf (szMess, szErrorString[9], szFile);
		if (!bOptions[OPT_Silent])
			ShowMessage (szMess);
	}
	pdw++;
	if (*pdw != formtypeAVI)
	{	strcpy (szErr, szErrorString[10]); return -4;	}
	pdw++;

	// Header analyse (streams search until data begins)

	totpos = nbs = 0;
    while ((*pdw != listtypeAVIMOVIE) &&	// "movi" = end of header
    	(*(DWORD *)((char *)pdw+2) != listtypeAVIMOVIE) &&
        	((char *)pdw - szBuf < len))
	{
		if (*pdw == ckidSTREAMHEADER)   // Stream Header
		{
			pdw++;
			size = *pdw++;				// 0x30 for old AVI, 0x38 for new one

			// Set to 0 to see clearly what is filled
			memset(&si[nbs], 0, sizeof(AVISTREAMINFO));
			// We copy only interesting data in AVISTREAMINFO structure
			memcpy (&si[nbs], pdw, size);

			// We jump after "strf" to copy the second part
			pdw = (DWORD *)((char *)pdw + size + 4);
			size = *pdw++;
			if (si[nbs].fccType == streamtypeVIDEO)
				memcpy (&si[nbs].rcFrame.right, pdw+1, 16);	// Width, Height, .., Codec
			else if (si[nbs].fccType == streamtypeAUDIO)
			{
				as = pai->bAStreamsNb++;
				memcpy (&wfx[as], pdw, sizeof (WAVEFORMATEX));	// WAVEFORMATEX
	            si2wfx = nbs - as;
			}
			nbs++;
			pdw = (DWORD *)((char *)pdw + size);
            if ((char *)pdw - szBuf > len)  // We have a small problem ...
                break;                  // Exit to not go out of szBuf bounds
		}

		if ((*pdw == ckidAVIPADDING) || // JUNK
			(*pdw == ckidSTREAMNAME))	// strn
		{
			pdw++;
			size = *pdw;

            if (size%2 == 1)	size++;	// Bug? with strn
//            if (size == 0x09)	size = 0x0A;	// Bug? with Nandub 1.0rc2 (strn)
//            if (size == 0x0D)	size = 0x0E;	// Bug? with NeoVideo92CF
			pdw = (DWORD *)((char *)pdw + size);
		}

		pdw++;

        pos = (int)((char *)pdw - szBuf);
        if (pos >= len-100)             // End of buffer
        {
            pos -= len-100;
            if (pos <= 0)   pos = 0;
#ifdef DEBUG
    		ShowMessage ("Buffer refill");
#endif
       		fseek (fh, -100, SEEK_CUR);
        	len = fread (szBuf, 1, MAXHEAD, fh);
        	if (!len)                   // End Of File
                break;
           	pdw = (DWORD *)(szBuf+pos);
			totpos += MAXHEAD;
        }
	}
    pai->dwFileHeadSize = totpos + (int)((char *)pdw - szBuf);

	fclose (fh);
	if (!nbs)
	{	strcpy (szErr, szErrorString[11]); return -5;	}
#endif

	// Store video information

	pasi = &si[0];						// Pointer on stream info
	if ((pasi->fccType != streamtypeVIDEO) &&
       	(pasi->fccType != streamtypeAUDIO) &&
       	(pasi->fccType != streamtypeDV))
		return -3;						// The 1st stream is not video

	for (int s=0 ; s<nbs ; s++)
    {
	 pasi = &si[s];						// Pointer on stream info
     switch(pasi->fccType)
     {
     case streamtypeVIDEO:
     case streamtypeDV:
		// We take the second fourCC which is more accurate than the first
		if (pasi->dwFormatChangeCount < 0x19FFFFFF)
				*(DWORD *)pai->szVC = pasi->fccHandler;	// 1st FourCC
		else	*(DWORD *)pai->szVC = pasi->dwFormatChangeCount;	// 2nd FourCC (recent AVI)
		pai->dwWidth = pasi->rcFrame.right;
		pai->dwHeight = pasi->rcFrame.bottom;
		pai->dwFramesNb = pasi->dwLength;
		// We store 1000 * fps to keep a 3 digits precision
		pai->dwFps = 1000 * ((double)pasi->dwRate) / ((double)pasi->dwScale);
        break;

     case streamtypeAUDIO:
		int	n = s-si2wfx;
		pai->a[n].dwASec = pasi->dwLength * pasi->dwScale/pasi->dwRate;
		pai->a[n].dwASnb = pasi->dwLength;
		pai->a[n].wAC = wfx[n].wFormatTag;
		pai->a[n].dwAAvgBPS = wfx[n].nAvgBytesPerSec;
		pai->a[n].dwASps = wfx[n].nSamplesPerSec;
		pai->a[n].bAChannels = wfx[n].nChannels;
	 }
	}
    
	return 0;
}

// Open an MPG file and retrieve main information
// -----------------------------------------------------------

int GetMPGinfo (LPCSTR szFile)
{
	MpegInfo MI;

    MI.Init (szFile);
    MI.ParseFile ();
    MI.Close ();

	return 0;
}

// Open an RM file and retrieve main information
// -----------------------------------------------------------

// Global properties

struct PROP
{
	WORD	wObjectVersion;
    DWORD	dwMaxBitrate;
    DWORD	dwAverageBitrate;			// Global=Audio+Video
    DWORD	dwMaxPacketSize;
    DWORD	dwAveragePacketSize;
    DWORD	dwNumberPackets;
    DWORD	dwDuration;
    DWORD	dwUnknown1;
    DWORD	dwUnknown2;
    DWORD	dwUnknown3;
    DWORD	dwUnknown4;
};

// General stream properties (MeDia PRoperties)

struct MDPR
{
	WORD	wObjectVersion;
    WORD	wStreamNumber;
    DWORD	dwMaxBitrate;
    DWORD	dwAverageBitrate;
    DWORD	dwMaxPacketSize;
    DWORD	dwAveragePacketSize;
	DWORD	dwStartTime;
    DWORD	dwPreroll;
    DWORD	dwDuration;
    BYTE	bStreamNameSize;
    char	szStreamName[0xC];		// 12 is a common value for bStreamNameSize
    BYTE	bMimeTypeSize;
    char	szMimeType[0x14];		// 20 is a common value for bMimeTypeSize
    DWORD	dwTypeSpecificLength;
    DWORD	dwChunkID;					// .ra0xFD for audio
    DWORD	dwVChunkID;					// VIDO for Video
};

// Specific audio stream properties

struct MDPR_Audio
{
//	DWORD	dwVersion;
	char	dwFourCC[4];
    DWORD	dwDummy1;
    WORD	wDummy2;
	DWORD	dwHeaderSize;
    WORD	wAudCodecInfo;		// Flavor ?
    DWORD	dwCodedFrameSize;
    DWORD	dwDummy3;
    DWORD	dwDummy4;
    DWORD	dwDummy5;
    WORD	wCodedFrameSize;	// ?
    DWORD	dwDummy6;
    DWORD	dwDummy7;
    WORD	wSampleRate;
    DWORD	dwDummy8;
    DWORD	dwDummy9;
    WORD	wChannels;
	char	dwGenr[4];			// always genr (?)
	char	dwAudCodec[4];		// cook, atrc, sipr
};

// Specific video stream properties

struct MDPR_Video
{
	char	dwFourCC[4];
	WORD	wWidth;
    WORD	wHeight;
	WORD	wDummy1;
	WORD	wDummy2;
	WORD	wDummy3;
    WORD	wFrameRate;
	WORD	wDummy4;
	DWORD	dwDummy5;
	DWORD	dwDummy6;
};

int GetRMinfo (LPCSTR szFile)
{
	FILE	*fh;
	char	szBuf[MAXHEAD+1],
            szTemp[10];
    int		pos,
			as;
    DWORD	len,
    		code,
			dwAAvgBPS,
			dwASec,
            size;
    struct PROP	*pProp;
    struct MDPR	*pMdpr;
	struct MDPR_Audio	*pAudio;
	struct MDPR_Video	*pVideo;

	// Header reading

	fh = fopen (szFile, "rb");
	if (fh == NULL)
	{	strcpy (szErr, szErrorString[7]);	return -1;	}

	len = fread (szBuf, 1, MAXHEAD, fh);
	if (!len)
	{	strcpy (szErr, szErrorString[7]);	return -1;	}

	pai->dwFileHeadSize = 0x374;
	pos = 0;
    while (TRUE)
	{
		code = *(DWORD *)(szBuf+pos);
        if (code == mmioFOURCC('D','A','T','A'))
        	break;						// End of header

		size = INVDW(*(DWORD *)(szBuf+pos+4));

		switch (code)
        {
        case mmioFOURCC('.','R','M','F'):	// Real Media File
        case mmioFOURCC('C','O','N','T'):
            break;

        case mmioFOURCC('P','R','O','P'):
            pProp = (struct PROP *)(szBuf+pos+8);
            if (pProp->wObjectVersion)
            	break;
            pai->dwFileDuration = INVDW (pProp->dwDuration) /1000;
            break;

        case mmioFOURCC('M','D','P','R'):
            pMdpr = (struct MDPR *)(szBuf+pos+8);
            if (pMdpr->wObjectVersion)
            	break;

			if (!strncmp (pMdpr->szStreamName, "Audio", 5))// Audio Stream
//                   if (pMdpr->dwChunkID == mmioFOURCC('.','r','a',0xFD))	// Audio Stream
			{
				pai->bAStreamsNb++;
				as = pai->bAStreamsNb-1;
				dwAAvgBPS = INVDW (pMdpr->dwAverageBitrate)/8;
				dwASec = INVDW (pMdpr->dwDuration) /1000;

				if (as < MAXAUDIOSTREAMS)
                {
					pai->a[as].dwASec = dwASec;

					pAudio = (struct MDPR_Audio *)(szBuf+pos+8+sizeof (struct MDPR)+pMdpr->bStreamNameSize-0xC+pMdpr->bMimeTypeSize-0x14);
					if (pAudio->dwFourCC[3] == '5')
						strcpy (pai->a[as].szACname, "RealAudio 8");
					else
					{
						strncpy (pai->a[as].szACname, pAudio->dwFourCC, 4);
						pai->a[as].szACname[4] = 0;
					}
					strcat (pai->a[as].szACname, ",");
					strncat (pai->a[as].szACname, pAudio->dwAudCodec, 4);
					strcat (pai->a[as].szACname, "-");
					strcat (pai->a[as].szACname, itoa (INVW (pAudio->wAudCodecInfo), szTemp, 10));

					pai->a[as].bAChannels = INVW (pAudio->wChannels);
					pai->a[as].dwASps = INVW (pAudio->wSampleRate);
					pai->a[as].dwAAvgBPS = dwAAvgBPS;
	                pai->a[as].dwASnb = pai->a[as].dwASps * dwASec;
					pai->a[as].wACsupp = ftSUP;	// It is supported with RealPlayer
				}
            }
			else if (!strncmp (pMdpr->szStreamName, "Video", 5))// Video Stream
//                  else if (pMdpr->dwVChunkID == mmioFOURCC('V','I','D','O'))
			{
				pai->dwVAvgBPS = INVDW (pMdpr->dwAverageBitrate)/8;
				pai->dwVSec = INVDW (pMdpr->dwDuration) /1000;
				pai->dwVSize = pai->dwVAvgBPS * pai->dwVSec;

				pVideo = (struct MDPR_Video *)(szBuf+pos+8+sizeof (struct MDPR)+pMdpr->bStreamNameSize-0xC+pMdpr->bMimeTypeSize-0x14);
//				*(DWORD *)pai->szVC = pVideo->dwFourCC;
				strncpy (pai->szVC, pVideo->dwFourCC, 4);
				pai->szVC[4] = 0;
				pai->dwWidth = INVW (pVideo->wWidth);
				pai->dwHeight = INVW (pVideo->wHeight);
				pai->dwFps = 1000*INVW (pVideo->wFrameRate);
				pai->wVCsupp = ftSUP;	// It is supported with RealPlayer
            }
            break;
        }
        pos += size;
	}
	pai->dwFileHeadSize = pos;

	fclose (fh);
	return 0;
}

// Open an OGG file and retrieve main information
// -----------------------------------------------------------
// http://www.xiph.org/ogg/vorbis/doc/vorbis-spec-ref.html
// http://tobias.everwicked.com/packfmt.htm

struct OGG_Page_Header
{
	BYTE	bStreamStructureVersion;	// At this time always 0
	BYTE	bHeaderTypeFlag;			// b0set = continued packet, b1set = 1st page of logical bitstream, b2set = last page of logical bitstream
	DWORD	dwGranulePosition;			// Low DW (GranulePosition is __int64, but HighDW is = 0 at this time)
	DWORD	dwGranulePositionHi; 	 	// High DW
	DWORD	dwStreamSerialNumber;		// Link between logical & physical bitstream
	DWORD	dwPageSequenceNumber;
	DWORD	dwPageChecksum;
	BYTE	bPageSegments;
};

struct OGG_Common_Header
{
	BYTE	bPacketType;				// 1=ident, 3=comment, 5=setup, b0=0->audio
	char	cName[6];					// vorbis / video / Direct
};

struct OGG_Vorbis
{
	DWORD	dwVorbisVersion;
	BYTE	bAudioChannels;
	DWORD	dwAudioSampleRate;			// Sps
	DWORD	dwBitrateMaximum;			// bps
	DWORD	dwBitrateNominal;
	DWORD	dwBitrateLower;
    BYTE	bBlockSize;					// 2 exponent [4bits] | 2 exponent [4bits]
    BYTE	bStopFlag;					// Always 1
};

//// OggDS headers
// Header for the new header format
typedef struct OGG_Tobias_video
{
    DWORD	dwWidth;
    DWORD	dwHeight;
} OGG_Tobias_video;

// Taken from http://tobias.everwicked.com/packfmt.htm

typedef struct OGG_Tobias_audio
{
	WORD	wAChannels;
	WORD	wABlockalign;
	DWORD	dwAvgBPS;
} OGG_Tobias_audio;

struct OGG_Tobias						// Equivalent to struct stream_header
{
	char	cNameEnd[2];				// In Tobias def, cName is 8 characters long
	char	cSubtype[4];
    DWORD	dwSize;						// size of the structure
    DWORD	dwTimeUnit;
    DWORD	dwTimeUnitHi;
    DWORD	dwSamplesPerUnit;
    DWORD	dwSamplesPerUnitHi;
    DWORD	dwDefaultLen;				// in media time
    DWORD	dwBufferSize;
    WORD	wBitsPerSample;
    WORD	wPadding;
	union
	{
		OGG_Tobias_video	video;		// Video specific
		OGG_Tobias_audio	audio;		// Audio specific
	} sh;
};

void GetData (void *variable, int size, char *buffer)
{
	memcpy (variable, buffer, size);
}

void GetDataPos (void *variable, int size, char *buffer, int &position)
{
	memcpy (variable, buffer+position, size);
	position += size;
}
/// Some defines from OggDS
#define PACKET_TYPE_HEADER       0x01
#define PACKET_TYPE_COMMENT      0x03
#define PACKET_TYPE_BITS         0x07

#define MAXOGGSTREAMS	10				// Its more 255, but I simplify ...

int GetOGGinfo (LPCSTR szFile)
{
	FILE	*fh;
	char	szBuf[MAXHEAD],
    		szSubtype[5],
            c,
            cStreamsType[MAXOGGSTREAMS];
    int		pos,
			as;
    DWORD	len,
    		time,
    		dwIdent,
            dwPageJump,
            dwPageSize;
	struct OGG_Page_Header	oph;
	struct OGG_Common_Header	och;
	struct OGG_Tobias	ot;
	struct OGG_Vorbis	oa;

	// Header reading

	fh = fopen (szFile, "rb");
	if (fh == NULL)
	{	strcpy (szErr, szErrorString[7]);	return -1;	}

	len = fread (szBuf, 1, MAXHEAD, fh);
	if (!len)
	{	strcpy (szErr, szErrorString[7]);	return -1;	}

	for (int i=0 ; i<MAXOGGSTREAMS ; i++)
    	cStreamsType[i] = ' ';

	pos = 0;
	GetDataPos (&dwIdent, sizeof (DWORD), szBuf, pos);
    while (dwIdent == mmioFOURCC('O','g','g','S'))
    {
		GetDataPos (&oph, sizeof (OGG_Page_Header), szBuf, pos);
        if (oph.bHeaderTypeFlag & 0x2)	// 1st page of logical bitstream
        {
            dwPageSize = 0;
            for (int i=0; i<oph.bPageSegments ; i++)
				dwPageSize += 0x000000FF & szBuf[pos++];
			dwPageJump = pos + dwPageSize;

			GetDataPos (&och, sizeof (OGG_Common_Header), szBuf, pos);
			if (och.bPacketType==PACKET_TYPE_HEADER)
			{
				if (!strncmp (och.cName, "video", 5))
				{
					GetDataPos (&ot, sizeof (OGG_Tobias), szBuf, pos);
                    strncpy(pai->szVC, ot.cSubtype, 4);
					pai->szVC[4] = '\0';

					pai->dwWidth = ot.sh.video.dwWidth;
					pai->dwHeight = ot.sh.video.dwHeight;
					pai->dwFps = ((double)10000000000) / ot.dwTimeUnit;

                    cStreamsType[oph.dwStreamSerialNumber] = 'v';
				}
				else if (!strncmp (och.cName, "audio", 5))
				{
					GetDataPos (&ot, sizeof (OGG_Tobias), szBuf, pos);
					pai->bAStreamsNb++;
					as = pai->bAStreamsNb-1;
                    cStreamsType[oph.dwStreamSerialNumber] = as;

					if (as < MAXAUDIOSTREAMS)
                    {
	                    strncpy(szSubtype, ot.cSubtype, 4);
						szSubtype[4] = '\0';
						pai->a[as].wAC = strtoul(szSubtype, NULL, 16);

						pai->a[as].bAChannels = ot.sh.audio.wAChannels;
						pai->a[as].dwASps = ot.dwSamplesPerUnit;
						pai->a[as].dwAAvgBPS = ot.sh.audio.dwAvgBPS * 8 / 8;
                    }
				}
            	else if (!strncmp (och.cName, "vorbis", 6))
				{
					GetDataPos (&oa, sizeof (OGG_Vorbis), szBuf, pos);
					as = pai->bAStreamsNb;
					pai->bAStreamsNb++;
					DWORD ssn = oph.dwStreamSerialNumber;
					if (ssn >= MAXOGGSTREAMS)	ssn = 0;	// For music files
                    cStreamsType[ssn] = as;

					pai->a[as].wAC = 0x674F;

					if (pai->bAStreamsNb <= MAXAUDIOSTREAMS)
                    {
						pai->a[as].bAChannels = oa.bAudioChannels;
						pai->a[as].dwASps = oa.dwAudioSampleRate;
                        if (oa.dwBitrateNominal != 0xFFFFFFFF)
							pai->a[as].dwAAvgBPS = oa.dwBitrateNominal / 8;	// Convert to Byte/s
                    }
				}
            	else if (!strncmp (och.cName, "Direct", 6))	// Direct Show
                {
					VIDEOINFOHEADER		pVIH;

					strncpy (pai->szVC, szBuf+pos+0x3D, 4);
					pai->szVC[4] = '\0';

					pos +=0x75;
					GetDataPos (&pVIH, sizeof (VIDEOINFOHEADER), szBuf, pos);
					pai->dwWidth = pVIH.bmiHeader.biWidth;
					pai->dwHeight = pVIH.bmiHeader.biHeight;
					pai->dwFps = ((double)10000000000) / pVIH.AvgTimePerFrame;
				}
            	else if (!strncmp (och.cName, "text", 4))
                {	}					// We don't do anything with comments
				else
                {
					if (!bOptions[OPT_Silent])
                    {
						char	szBuf[200];
						wsprintf (szBuf, szErrorString[12], och.cName, szFile);
						ShowMessage (szBuf);
                    }
                }
            }
        }
        else break;						// End of header information ?
		pos = dwPageJump;				// pos should already be correct
		GetDataPos (&dwIdent, sizeof (DWORD), szBuf, pos);	// Should be OggS
    }
	pai->dwFileHeadSize = pos;

	// Now we read the end of file to read the last packet and calculate stream time

	fseek (fh, -MAXHEAD, SEEK_END);
	len = fread (szBuf, 1, MAXHEAD, fh);
	if (!len)
	{	strcpy (szErr, szErrorString[13]);	return -1;	}

	pos = MAXHEAD-20;
    do
    {
        while ((*(DWORD *)&szBuf[pos] != mmioFOURCC('O','g','g','S')) && pos)
        	pos--;
		if (!pos)						// No packet found, should not be possible
        	break;

		GetData (&oph, sizeof (OGG_Page_Header), szBuf+pos+4);

		DWORD ssn = oph.dwStreamSerialNumber;
		if (ssn >= MAXOGGSTREAMS)	ssn = 0;	// For music files
		c = cStreamsType[ssn];
        if (c == ' ')
        	continue;
        if (c == 'v')
				time = (1000L * oph.dwGranulePosition + pai->dwFps/2) / pai->dwFps;
        else if ((c == 0) || (c == 1) || (c == 2))
        	time = (oph.dwGranulePosition + pai->a[c].dwASps/2) / pai->a[c].dwASps;
		else continue;					// Audio stream unknown

		pai->dwFileDuration = pai->a[0].dwASec = pai->dwVSec = time;
        break;
	}	while (TRUE);

	fclose (fh);
	return 0;
}

// Test if readable by DirectShow
// In detailled mode, some info on the filetype is searched
// -----------------------------------------------------------

#define MAXFILTERS	10
#define MAXPINS		4

int DSHOWtest (LPCSTR szFile, BOOL bDetailled)
{
	HRESULT hr;
	wchar_t	wszFile[MAXPATH];

	// Create the Filter Graph Manager.
	IGraphBuilder *pGraph = 0;
	hr = CoCreateInstance(CLSID_FilterGraph, 0,	CLSCTX_INPROC_SERVER,
    	IID_IGraphBuilder,	reinterpret_cast<void**>(&pGraph));
	if (FAILED(hr))
	{	AMGetErrorText(hr, szErr, SZERR); return -1;	}

	StringToWideChar (szFile, wszFile, MAXPATH);
	hr = pGraph->RenderFile (wszFile, 0);
	if ((hr == S_OK) || (hr == VFW_S_PARTIAL_RENDER))
	{
		// Analyse the filters
		IEnumFilters *pEnumF = 0;
		IBaseFilter *pF[MAXFILTERS], *cF;
        FILTER_INFO FilterInfo, cFI;
		IPin	*pP[MAXPINS], *cP;
		PIN_INFO PinInfo, cPI;
		ULONG nbF, nbP, nSplitter;
		char szName[MAXPATH], *p;
		AM_MEDIA_TYPE MType;

		pGraph->EnumFilters (&pEnumF);
		memset (pF, 0, MAXFILTERS * sizeof (IBaseFilter *));
		pEnumF->Next(MAXFILTERS, pF, &nbF);
		// We consider that 4 filters max means audio file
		pai->bFileAudio = (nbF<=4);

		if (nbF)
        {
			// If ASF filetype, we know that we have to study the last filter
			if (pai->iFileType == ftASF)
            {
				IEnumPins *pEnumP = 0;
				pF[nbF-1]->EnumPins (&pEnumP);
				memset (pP, 0, MAXPINS * sizeof (IPin *));
				pEnumP->Next(MAXPINS, pP, &nbP);
                pEnumP->Release();
            }

			// First, we search the AVI Splitter (audio&video) filter
            // to try obtain the file format
            // Notice : the filters are in reverse order, nbF-1 = last one
            // i=1 to avoid the "Filename"
			else for (UINT i=1 ; i<=2 ; i++)
            {
				IEnumPins *pEnumP = 0;
				nSplitter = nbF-1-i;

				pF[nSplitter]->EnumPins (&pEnumP);
				memset (pP, 0, MAXPINS * sizeof (IPin *));
				pEnumP->Next(MAXPINS, pP, &nbP);

				if (bDetailled)
        	    {
	    		    hr = pF[nSplitter]->QueryFilterInfo(&FilterInfo);
					strcpy (szName, WideCharToString(FilterInfo.achName).c_str());
	    	        if (strstr (szName, "MPEG"))	// For "Ligos MPEG splitter"
    	    	    	strcpy (pai->szFileType, "MPEG");
        	    	else
	        	    {
						p = strchr (szName, ' ');
                        if (p && !strchr(szName, '\\'))	// Seems to be a valid filter
                        {
							*p = '\0';
    				   		strcpy (pai->szFileType, szName);
                        }
					}
		        	FilterInfo.pGraph->Release();
        	    }

                pEnumP->Release();
				if (nbP == i+2-pai->bFileAudio)	// (2 for 1st filter, 3 for 2nd filter)
               		break;
			}

			// Then we study output pin to find audio & video filter type

			for (int i=nbP-1 ; i>=0 ; i--)
			{
				pP[i]->QueryPinInfo(&PinInfo);
				strcpy (szName, WideCharToString(PinInfo.achName).c_str());
				pP[i]->ConnectedTo(&cP);
                if (PinInfo.dir && cP)	// Output & connected pin
                {
					pP[i]->ConnectionMediaType(&MType);
					cP->QueryPinInfo(&cPI);
    	            cF = cPI.pFilter;
	    		    hr = cF->QueryFilterInfo(&cFI);
					strcpy (szName, WideCharToString(cFI.achName).c_str());
					p = strrchr (szName, ' ');
					if (p)	*p = '\0';

					if (MType.majortype == MEDIATYPE_Video)
                    {
						if (bDetailled)
			            {
							// FourCC GUID
							if (!MType.subtype.Data2 && (MType.subtype.Data3==16))
									*(DWORD *)pai->szVC = MType.subtype.Data1;
							else	strcpy (pai->szVCname, szName);
                        }
                        pai->wVCsupp = ftSUP;
                    }
	                else if (MType.majortype == MEDIATYPE_Audio)
    	            {
						int as = 0;
						if (bDetailled)
                        {
							as = pai->bAStreamsNb++;
	    				   	strcpy (pai->a[as].szACname, szName);
                        }
                        pai->a[as].wACsupp = ftSUP;
					}
	                else if (MType.majortype == MEDIATYPE_Midi)
    	            {
						int as = 0;
						if (bDetailled)
                        {
							as = pai->bAStreamsNb++;
	    				   	strcpy (pai->a[as].szACname, "Midi");
                        }
                        pai->a[as].wACsupp = ftSUP;
					}

					cFI.pGraph->Release();
					cPI.pFilter->Release();
					cP->Release();
//					CoTaskMemFree(pMType);	// Seem Unnecessary, cause a bug in ntdll.dll
				}
		        PinInfo.pFilter->Release();
				pP[i]->Release();
			}

			for (int i=nbF-1 ; i>=0 ; i--)
				pF[i]->Release();
		}
		pEnumF->Release();
	}

	if (hr)
	{	AMGetErrorText(hr, szErr, SZERR); return -1;	}

	pGraph->Release();
	return hr;							// S_OK (0) if ok
}

// Try to obtain file info using DirectShow (available from DX8)
// -----------------------------------------------------------

int GetDSHOWinfo (LPCSTR szFile)
{
	BOOL	bFirstAudio;
	HRESULT hr;
	IMediaDet *pDet;
	wchar_t	wszFile[MAXPATH];
    long Streams = 0;
//    long VideoStream = -1;
	AM_MEDIA_TYPE MType;

	// Create the System Device Enumerator.
	pDet = NULL;
	hr = CoCreateInstance(CLSID_MediaDet, NULL, CLSCTX_INPROC_SERVER,
		IID_IMediaDet, (void **)&pDet);
	if (FAILED(hr))
	{	AMGetErrorText(hr, szErr, SZERR); return -1;	}

	StringToWideChar (szFile, wszFile, MAXPATH);
    hr = pDet->put_Filename (wszFile);
	if (FAILED(hr))
	{	AMGetErrorText(hr, szErr, SZERR);	return -1;	}

    // Find the video and audio streams

	bFirstAudio = true;
    hr = pDet->get_OutputStreams (&Streams);
    for (int i = 0 ; i < Streams ; i++)
    {
		double	sec;

        pDet->put_CurrentStream (i);
		memset (&MType, 0, sizeof(MType));
	    hr = pDet->get_StreamMediaType (&MType);
	    hr = pDet->get_StreamLength (&sec);

        if (MType.majortype == MEDIATYPE_Video)
		{
			VIDEOINFOHEADER		*pVIH;
			VIDEOINFOHEADER2	*pVIH2;
			GUID	ft = MType.formattype;

//			VideoStream = i;

			pai->dwVSec = (DWORD)sec;
			if ((ft==FORMAT_MPEGVideo) || (ft==FORMAT_VideoInfo))
            {
				pVIH = (VIDEOINFOHEADER *) MType.pbFormat;
				pai->dwWidth = pVIH->bmiHeader.biWidth;
				pai->dwHeight = pVIH->bmiHeader.biHeight;
				pai->dwFps = ((double)10000000000) / pVIH->AvgTimePerFrame;
			}
			else if ((ft==FORMAT_MPEG2Video) || (ft==FORMAT_VideoInfo2))
			{
				pVIH2 = (VIDEOINFOHEADER2 *) MType.pbFormat;
				pai->dwWidth = pVIH2->rcSource.right;
				pai->dwHeight = pVIH2->rcSource.bottom;
				pai->dwVAvgBPS = pVIH2->dwBitRate/8;
            }
        }
		else if (MType.majortype == MEDIATYPE_Audio)
        {
			WAVEFORMATEX		*pWF;
			if (MType.formattype == FORMAT_WaveFormatEx)
			{
				if (bFirstAudio)
				{
					pai->bAStreamsNb = 0;	// In case of already computed in DSHOWtest()
					bFirstAudio = false;
				}
				int	as = pai->bAStreamsNb++;

				pai->a[as].dwASec = (DWORD)sec;
				pWF = (WAVEFORMATEX *) MType.pbFormat;
				pai->a[as].wAC = pWF->wFormatTag;
				strcpy (pai->a[as].szACname, "");	// To allow obtaining a correct name
				pai->a[as].dwAAvgBPS = pWF->nAvgBytesPerSec;
				pai->a[as].dwASps = pWF->nSamplesPerSec;
				pai->a[as].bAChannels = pWF->nChannels;
			}
        }
    }

	pDet->Release();
	// Returns an error if no video stream found
//	return (VideoStream != -1 ? 0 : VFW_E_INVALIDMEDIATYPE);
	return 0;
}

// Calculate a FileQuality ratio (percentage)
// -----------------------------------------------------------

WORD CalcFQ ()
{
	WORD	fq,
    		fps,
    		bps;
    int		val;

	// 20% on Video FrameRate. Min = 20 fps, Max = 30 fps
    fps = pai->dwFps / 1000;
    if (fps)
    {
		val = 20 * (fps - 20) / (30 - 20);
    	if ((val < 0) || (fps < 20))	val=0;
	    if ((val > 20) || (fps > 30))	val=20;
    	fq = val;
    }
    else	fq = 0;

	// 30% on Video Resolution. Min=TV 160*120, Med=VCD 320*240, Max=640*480
    // To ease calculation (to not use sqrt (), only Width is taken
	val = 30 * (pai->dwWidth - 160) / (640 - 160);
    if (pai->dwWidth)
    {
	    if ((val < 0) || (pai->dwWidth < 160))	val=0;
    	if ((val > 30) || (pai->dwWidth > 640))	val=30;
	    fq += val;
	}

	// 30% on Video BitRate. Min = 500 Kbps, Max = 2000 Kbps
    if (pai->dwVAvgBPS)
    {
		bps = pai->dwVAvgBPS * 8 / 1000;
		val = 30 * (bps - 500) / (2000 - 500);
    	if ((val < 0) || (bps < 500))	val=0;
	    if ((val > 30) || (bps > 2000))	val=30;
    }
    else if (fq)	val=15;				// Default value if not defined
    else	val = 0;
    fq += val;

	// 20% on Audio BitRate. Min = 96 Kbps, Med= 128 Kbps, Max = 192 Kbps
	if (fq)								// Video file
    {
	    if (pai->a[0].dwAAvgBPS)
    	{
			bps = pai->a[0].dwAAvgBPS * 8 / 1000;
			val = 20 * (bps - 96) / (192 - 96);
    		if ((val < 0) || (bps < 96))	val=0;
	    	if ((val > 20) || (bps >192))	val=20;
	    }
    	else val=10;					// Default value if not defined
	    fq += val;
    }
    else
    {
		// 100% on Audio BitRate. Min = 5 Kbps, Med= 32 Kbps, Max = 64 Kbps
		bps = pai->a[0].dwASps / 1000;
		val = 100 * (bps - 5) / (64 - 5);
    	if (val < 0)	val=0;
	    if (val > 100)	val=100;
		fq = val;
    }

    return fq;
}

// Completion of the data structure from the existing elements
// -----------------------------------------------------------

int FileInfoFill ()
{
	int	n = min (pai->bAStreamsNb, MAXAUDIOSTREAMS);

	// Find if the video codec is supported by the system

	if (*pai->szVC && (pai->wVCsupp==ftUNS))
    {
		ICINFO		icinfo;

		pai->wVCsupp = ICInfo (ICTYPE_VIDEO, *(DWORD *)pai->szVC, &icinfo) ? ftSUP : ftUNS;
		if (pai->wVCsupp==ftSUP)
			strcpy (pai->szVCdriver, WideCharToString (icinfo.szDriver).c_str());
    }

	// Search detailled name and download url for this video codec

	if (!*pai->szVCname && *pai->szVC)
		strcpy (pai->szVCname, searchVCname (pai->szVC));
	if (!*pai->szVCurl && *pai->szVC)
		strcpy (pai->szVCurl, searchVCurl (pai->szVC));

	// Complete video info

	if (!pai->wAspRatio && pai->dwHeight)// To avoid possible "Divide per zero" error
		pai->wAspRatio = 100 * pai->dwWidth / pai->dwHeight;
	if (!*pai->szAspRatio && pai->wAspRatio)
		strcpy (pai->szAspRatio, searchAspRatio (pai->wAspRatio));

	// Audio calculation (before video because more info in AVI)

	if (!pai->a[0].dwASec && pai->dwFileDuration)	// Needed for ASF
		pai->a[0].dwASec = pai->dwFileDuration;
	// If 1 audio stream but incorrect (>3 hours) & video time available
	if ((n==1) && (pai->a[0].dwASec > 3*3600) && pai->dwFramesNb && pai->dwFps)
		pai->a[0].dwASec = pai->dwFramesNb * 1000 / pai->dwFps;

	pai->dwATotSize = 0;
	for (int as=0 ; as<n ; as++)
	{
		// Find if the audio codec is supported by the system

		if (pai->a[as].wAC && (pai->a[as].wACsupp==ftUNS))
	    {
			ACMFORMATTAGDETAILS aftd;

			memset (&aftd, 0, sizeof (ACMFORMATTAGDETAILS));
			aftd.cbStruct = sizeof (ACMFORMATTAGDETAILS);
			aftd.dwFormatTag = pai->a[as].wAC;
			pai->a[as].wACsupp = !acmFormatTagDetails (NULL, &aftd, ACM_FORMATTAGDETAILSF_FORMATTAG) ? ftSUP : ftUNS;
// 10/02/2003 : I prefer the original name (ASF)
			if ((pai->a[as].wACsupp==ftSUP) && !*pai->a[as].szACname)	// We ask the driver for the codec name
				strcpy (pai->a[as].szACname, aftd.szFormatTag);
	    }

		// Search detailled name and download url for this audio codec

		if (!*pai->a[as].szACname && pai->a[as].wAC)
			strcpy (pai->a[as].szACname, searchACname (pai->a[as].wAC));
		if (!*pai->a[as].szACurl && pai->a[as].wAC)
			strcpy (pai->a[as].szACurl, searchACurl (pai->a[as].wAC));

		if (!pai->a[as].dwASec && pai->a[0].dwASec)
           	pai->a[as].dwASec = pai->a[0].dwASec;
		if (!pai->a[as].dwASize && pai->a[as].dwAAvgBPS && pai->a[as].dwASec)
	    {
			pai->a[as].dwASize = pai->a[as].dwAAvgBPS * pai->a[as].dwASec;
			pai->dwATotSize += pai->a[as].dwASize;
        }
		if (!pai->a[as].dwASnb && pai->a[as].dwASec && pai->a[as].dwASps)
           	pai->a[as].dwASnb = pai->a[as].dwASec * pai->a[as].dwASps;
    }
	if (n < pai->bAStreamsNb)		// We estimate the size to be the same as the 1st audio stream
		for (int i=n ; i<pai->bAStreamsNb ; i++)
            pai->dwATotSize += pai->a[0].dwASize;

	if (!pai->dwVSec && pai->dwFramesNb && pai->dwFps)
		pai->dwVSec = pai->dwFramesNb * 1000 / pai->dwFps;
	if (!pai->dwVSec && pai->dwFileDuration)
		pai->dwVSec = pai->dwFileDuration;
	if (!pai->dwFileDuration && pai->dwVSec)
		pai->dwFileDuration = pai->dwVSec;
	if (!pai->a[0].dwASec && pai->dwVSec)
		pai->a[2].dwASec = pai->a[1].dwASec = pai->a[0].dwASec = pai->dwVSec;

	// Video calculation

	if (!pai->dwVSize && pai->dwVAvgBPS && pai->dwVSec)
		pai->dwVSize = pai->dwVAvgBPS * pai->dwVSec;
	if (!pai->dwVSize && pai->dwFileTheoSize && pai->dwATotSize)
		pai->dwVSize = pai->dwFileTheoSize - pai->dwFileHeadSize - pai->dwATotSize;
	if (!pai->dwVSize && pai->dwFileRealSize && pai->dwATotSize && (pai->dwFileRealSize > pai->dwATotSize))
		pai->dwVSize = pai->dwFileRealSize - pai->dwFileHeadSize - pai->dwATotSize;
	if (!pai->dwVSize && !pai->bAStreamsNb) // No audio
		pai->dwVSize = (pai->dwFileTheoSize ? pai->dwFileTheoSize :
        				pai->dwFileRealSize) - pai->dwFileHeadSize;

	if (!pai->dwVAvgBPS && pai->dwVSize && pai->dwVSec)
		pai->dwVAvgBPS = pai->dwVSize / pai->dwVSec;

	if (!pai->dwFramesNb && pai->dwFps && pai->dwVSec)
		pai->dwFramesNb = pai->dwFps * pai->dwVSec / 1000;

	if (!pai->dwFps && pai->dwFramesNb && pai->dwVSec)
		pai->dwFps = 1000 * pai->dwFramesNb / pai->dwVSec;

	if (!pai->dwFileTheoSize && pai->dwVSize && pai->dwATotSize)	// Debug purpose
		pai->dwFileTheoSize =  pai->dwFileHeadSize + pai->dwVSize + pai->dwATotSize;

	pai->wFileQuality = CalcFQ ();

	return 0;
}

// Find the type of multimedia file
// by reading the 1st 4 bytes
// Return 0 for AVI, 1 for MPG, 2 for OGG, 3 for Real, 4 for DirectShow
// 5 for ASF
// -----------------------------------------------------------

#define SMALLHEADSIZE	0x80

filetype FileType (LPCSTR szFile, BOOL bSaveHeader)
{
	FILE	*fhout;
	char	*bfr,
    		szBuf[SMALLHEADSIZE],
    		szOutFile[255];
	DWORD	*pdw;
	int		pos,
		    dwFileSize;
	BOOL	bTest;
	SYSTEMTIME	st;
	HANDLE	hf;
	BY_HANDLE_FILE_INFORMATION	bhfi;
	DWORD	l;
	LARGE_INTEGER	qwSize;

	pai->iFileType = ftUNK;
	// Use of CreateFile to enable very big files access (> 4 Gb)
	hf = CreateFile (szFile, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hf == INVALID_HANDLE_VALUE)
	{
		LPSTR lpMsgBuf;

		FormatMessage(
		    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		    NULL,
		    GetLastError(),
		    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		    (LPTSTR) &lpMsgBuf,
		    0,
		    NULL
		);
//    	strcpy (szErr, szErrorString[7]);
    	strcpy (szErr, lpMsgBuf);
		LocalFree( lpMsgBuf );
        return ftUNK;
    }

	// Read file size
	GetFileInformationByHandle (hf, &bhfi);
	pai->dwFileRealSize = bhfi.nFileSizeLow;
	qwSize.LowPart = bhfi.nFileSizeLow;
	qwSize.HighPart = bhfi.nFileSizeHigh;
	pai->dwFileRealSizeKb = qwSize.QuadPart>>10;

	// Read file date and convert it to locale format
	FileTimeToSystemTime (&bhfi.ftLastWriteTime, &st);
	GetDateFormat (LOCALE_USER_DEFAULT, DATE_SHORTDATE, &st, NULL, pai->szFileDate, 11);

	ReadFile (hf, szBuf, SMALLHEADSIZE, &l, NULL);

    pos = 0;
	pdw = (DWORD *)szBuf;
	bTest = TRUE;
	while (bTest)
	{
		switch (*pdw)
		{
		case mmioFOURCC('R','I','F','F'):
    		pdw += 2;
        	if (*pdw == mmioFOURCC('A','V','I',' '))
	        {
				pai->iFileType = ftAVI;
				strcpy (pai->szFileType, "AVI");
				strcpy (pai->szFileUrl, "http://www.corecodec.com/modules.php?op=modload&name=Downloads&file=index&req=viewdownload&cid=1");
				bTest = FALSE;
	        }
    	    else if (*pdw == mmioFOURCC('C','D','X','A'))	// Container
			// Explanations on CDXA format :	// For in general MPG files
            // http://xcd.sourceforge.net/
			// http://www.wsu.edu/~benp/mcfcd.htm
			// Source code of cdxa2mpeg.c included in :
            // http://www.vcdimager.org
//	    		pdw += 15;
				bTest = FALSE;	// I force the end without FileType to allow a detection by DirectShow
			break;
		case INVDW(MPEG_PACK_START_CODE):// MPEG System
		case INVDW(MPEG_SEQUENCE_CODE):	// MPEG Video
			pai->iFileType = ftMPG;
			strcpy (pai->szFileType, "MPG");
			strcpy (pai->szFileUrl, "http://www.intervideo.com/jsp/Product_Download.jsp?p=WinDVD");
			bTest = FALSE;
			break;
		case mmioFOURCC('O','g','g','S'):
			pai->iFileType = ftOGG;
			strcpy (pai->szFileType, "OGG");
			strcpy (pai->szFileUrl, "http://www.corecodec.com/modules.php?op=modload&name=Downloads&file=index&req=viewdownload&cid=1");
			bTest = FALSE;
			break;
		case mmioFOURCC('.','R','M','F'):
			pai->iFileType = ftRMF;
			strcpy (pai->szFileType, "RMF");
			strcpy (pai->szFileUrl, "http://www.real.com/realone/");
			bTest = FALSE;
			break;
		case mmioFOURCC(0x30,0x26,0xB2,0x75):
			pai->iFileType = ftASF;
			strcpy (pai->szFileType, "ASF");
			strcpy (pai->szFileUrl, "http://www.microsoft.com/windows/windowsmedia/download/default.asp");
			bTest = FALSE;
			break;
		default:
			pos++;
            if (pos == SMALLHEADSIZE)
				bTest = FALSE;
			pdw = (DWORD *)&szBuf[pos];
		}
	}
	if (pai->iFileType == ftUNK)
    {
		if (!DSHOWtest (szFile, true))	// Playable with DirectShow
        {
			if (!stricmp(&szFile[strlen(szFile)-3], "mp3"))
            {
				pai->iFileType = ftMP3;
				strcpy (pai->szFileType, "MP3");
				strcpy (pai->szFileUrl, "http://www.microsoft.com/windows/windowsmedia/players.asp");
            }
            else
            {
				pai->iFileType = ftDSH;
				strcpy (pai->szFileType, "DSH");
				strcpy (pai->szFileUrl, "http://www.microsoft.com/windows/windowsmedia/players.asp");
            }
        }
//        else	strcpy (szErr, "Unknown format");
    }

	// Save header file for future reference

	if (bSaveHeader && !strstr (szFile, ".hdr"))
    {
		strcpy (szOutFile, szFile);
	    strcat (szOutFile, ".hdr");
		fhout = fopen (szOutFile, "wb");
		if (fhout == NULL)
        {
			wsprintf(szBuf, szErrorString[18], szOutFile);
			ShowMessage(szBuf);
        }
        else
        {
			dwFileSize = (pai->iFileType == ftMPG ? MAXSAVEDHEADER : 2*MAXHEAD);
			try {	bfr = new char [dwFileSize];	}
			catch (...)
			{	ShowMessage(szErrorString[0]); return pai->iFileType;		}

            SetFilePointer(hf, 0, NULL, FILE_BEGIN);
			ReadFile (hf, bfr, dwFileSize, &l, NULL);
			fwrite (bfr, 1, dwFileSize, fhout);// Write BOF

            SetFilePointer(hf, -dwFileSize, NULL, FILE_END);
			ReadFile (hf, bfr, dwFileSize, &l, NULL);
			fwrite (bfr, 1, dwFileSize, fhout);// Write EOF

			fclose (fhout);
			delete bfr;
	    }
	}
	CloseHandle (hf);

	return pai->iFileType;
}

