#include <vcl.h>
#pragma hdrstop

#pragma warn -pin						// Suppr warning 'Init is partially bracketed'
#pragma package(smart_init)
#pragma resource "*.dfm"

#define __MFC_COMPAT__

#include <vcl\Clipbrd.hpp>
#include <dir.h>						// findfirst
#include <dos.h>						// FA_DIREC
#include <stdio.h>						// fopen
#include <vfw.h>						// ACMDRIVERDETAIL
#include <ShlObj.h>

#include "Main.h"
#include "Options.h"
#include "About.h"
#include "fileinfo.h"
#include "multinfo.h"

// -----------------------------------------------------------
// Global data
// -----------------------------------------------------------

TMainForm *MainForm;
HWND hWndNextViewer=NULL;				// Clipboard viewer

struct FileInfo *pai;					// Allocated here, filled in FileInfo.cpp

struct COLUMN col[MAXCOL]
=
{
FNAM, NULL,260, 'A',
VFCC, NULL,60, 'A',
VCON, NULL,170, 'A',
VCOS, NULL,90, 'A',
VRES, NULL,60, 'N',
VRAT, NULL,60, 'N',
VFRQ, NULL,60, 'N',
VTIM, NULL,60, 'A',
VARA, NULL,80, 'A',
VSIZ, NULL,60, 'N',
ASTR, NULL,70, 'N',
ATSZ, NULL,60, 'N',
A1COI, NULL,70, 'A',
A1CON, NULL,150, 'A',
A1COS, NULL,90, 'A',
A1TIM, NULL,60, 'A',
A1CHA, NULL,60, 'N',
A1SNB, NULL,60, 'N',
A1FRQ, NULL,60, 'N',
A1RAT, NULL,60, 'N',
A1SIZ, NULL,60, 'N',
A2COI, NULL,70, 'A',
A2CON, NULL,130, 'A',
A2COS, NULL,90, 'A',
A2TIM, NULL,60, 'A',
A2CHA, NULL,60, 'N',
A2SNB, NULL,60, 'N',
A2FRQ, NULL,60, 'N',
A2RAT, NULL,60, 'N',
A2SIZ, NULL,60, 'N',
FTYP, NULL,60, 'A',
FRSZ, NULL,80, 'N',
FTSZ, NULL,80, 'N',
FQUA, NULL,60, 'N',
FDAT, NULL,60, 'A',
};

BOOL	bOptions[10];

__fastcall TMainForm::TMainForm(TComponent* Owner)
	: TForm(Owner)
{
	int pc = ParamCount();
    InitCommonControls();	// Loads COMCTL32.DLL

#ifdef USEAVIFUNC
	AVIFileInit ();
#endif

	Application->HintHidePause = 4000;	// 2500 par dfaut (2.5 sec)

	// Load parameters saved in registry
	bool	bInstalled = LoadParam ();

	// Load language texts
	if (!*szLang)
    {
		WORD lgid = PRIMARYLANGID (LANGIDFROMLCID (GetUserDefaultLCID ()));

		// include/winnt.h
		strcpy (szLang, lgid == LANG_CHINESE ? "Chinese simplified" :
        				lgid == LANG_CZECH ? "Czech" :
        				lgid == LANG_DUTCH ? "Nederlands" :
        				lgid == LANG_FRENCH ? "Francais" :
						lgid == LANG_GERMAN ? "Deutsch" :
						lgid == LANG_ITALIAN ? "Italiano" :
						lgid == LANG_HUNGARIAN ? "Hungarian" :
						lgid == LANG_POLISH ? "Polski" :
                        lgid == LANG_PORTUGUESE ? "Portugus" :
						lgid == LANG_ROMANIAN ? "Romana" :
						lgid == LANG_RUSSIAN ? "Russian" :
						lgid == LANG_SPANISH ? "Espaol" :
							        		"English");
    }

	if (InitText () == -1)
    {
		ShowMessage (szErr);
		exit (-1);
	}

    if ((pc == 1) && (ParamStr(1)=="/u"))	// If uninstall -> Registry cleaning
    {
		OptionsForm->UpdateAssoc(0, false);
		OptionsForm->UpdateAssoc(1, false);
		exit (-1);						// and exit
    }

	if (*szOtherString[10] != '-')	// If filter is defined in language string
		strcpy (szFilter, szOtherString[10]);	// I use it

	// If first installation, I set the association with files & folders
	if (!bInstalled)
    {
		OptionsForm->UpdateAssoc(0, true);
		OptionsForm->UpdateAssoc(1, true);
	}

	// Load codec information from file
	if (InitCodecs (szInstallDir) == -1)// Codec file not found
    {
		ShowMessage (szErr);
		exit (-1);
    }

	// COM init for DirectX access
	CoInitialize(NULL);

	// Listview columns init
	InitColumns ();

	// If there is 1 parameter on commandline, analyze it

    if (pc == 1)
    {
		ExecCmdLine(ParamStr (1).c_str());
		PostMessage(Handle, WM_AC_SELECT, 0, 0);	// Select 1st line
    }
}

__fastcall TMainForm::~TMainForm()
{
	CoUninitialize();
	SaveParam ();
}

// Execute command line parameters

void __fastcall TMainForm::ExecCmdLine(char *lpCmdLine)
{
	int		a;

	a = GetFileAttributes(lpCmdLine);
	if (!*lpCmdLine || (a == 0xFFFFFFFF))	// No CmdLine or error with file
    	return;

	strcpy(szInitDir, lpCmdLine);	// Because lpCmdLine may be limited in size
	if (a & FILE_ATTRIBUTE_DIRECTORY)	// It is a directory
		ScanDir (szInitDir, ChScanRec->Checked, false);
	else AddFile (szInitDir, false);	// It is a file

	if (LV->Items->Count)
        LV->Items->Item[ligIndex]->Selected = true;

	BringWindowToTop(Handle);	// In case it has disappear
}

// Init of colums
// -----------------------------------------------------------

void __fastcall TMainForm::InitColumns()
{
	for (int i=0 ; i<NBCOL ; i++)
		if (colId[i] >= 0)
		{
			LV->Columns->Items[i]->Caption = AnsiString (col[colId[i]].szColName);
			LV->Columns->Items[i]->Width = col[colId[i]].iColWidth;
		}
        else
		{
			LV->Columns->Items[i]->Caption = "";
			LV->Columns->Items[i]->Width = 0;
		}
}

void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
#ifdef USEAVIFUNC
	AVIFileExit ();
#endif
}

// Clear List
// -----------------------------------------------------------

void __fastcall TMainForm::ClearList()
{
	int	n = LV->Items->Count;
	while (n > 0)
	{
		TListItem *p = LV->Items->Item[--n];
		delete p->Data;
		p->Delete ();
	}
}

// File select
// -----------------------------------------------------------

void __fastcall TMainForm::SelFileBtnClick(TObject *Sender)
{
	OpenDialog1->Filter = szFilter;
	OpenDialog1->InitialDir = szInitDir;
	if (!OpenDialog1->Execute())
		return;
	GetCurrentDirectory (MAXPATH, szInitDir);

	if (ChClearList->Checked)
		ClearList();

	if (!AddFile (OpenDialog1->FileName.c_str(), ChSaveHeader->Checked))
        LV->Items->Item[ligIndex]->Selected = true;
}

// Select directory code
// Extracted from TBffolder by Torsten Johann (t-johann@gmx.de)

int __stdcall BffolderCallbackProc (HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	TMainForm	*TMF = (TMainForm *)lpData;

	if (uMsg == BFFM_INITIALIZED)
	{
		SetWindowText (hwnd, szMainForm[41]);	// Caption
//		SetWindowText (hwnd, "AVIcodec");	// Caption
		SendMessage (hwnd, BFFM_ENABLEOK, 0, TRUE);
        SendMessage (hwnd, BFFM_SETSELECTION, true, (LPARAM)TMF->szInitDir);
    }
	return 0;
}

void __fastcall TMainForm::SelDirBtnClick(TObject *Sender)
{
    LPMALLOC		lpMalloc;
    LPSHELLFOLDER	lpShellFolder;
    BROWSEINFO		bi;
    LPITEMIDLIST	ItemID;

	// We initialize the SHBrowseForFolder function

	if (SHGetMalloc (&lpMalloc) != NOERROR)
		return;
    if (SHGetDesktopFolder (&lpShellFolder) != NOERROR)
		return;

	// We clear the list

	if (ChClearList->Checked)
		ClearList();

	ZeroMemory (&bi, sizeof (BROWSEINFO));
	bi.ulFlags += BIF_RETURNONLYFSDIRS;

	bi.hwndOwner = Application->Handle;
	bi.pszDisplayName = szInitDir;
	bi.lpszTitle = szMainForm[57];
	bi.lpfn = BffolderCallbackProc;
	bi.lParam = (long)this;

	ItemID = SHBrowseForFolder (&bi);	// Display the search directory dialog

	lpShellFolder->Release ();
	if (ItemID != NULL)
	{
		SHGetPathFromIDList (ItemID, szInitDir);
		lpMalloc->Free (ItemID);
		lpMalloc->Release ();

		ScanDir (szInitDir, ChScanRec->Checked, false);
		if (LV->Items->Count)
	        LV->Items->Item[0]->Selected = true;
	}
}

// Add a file in the list, return 0 if ok
// -----------------------------------------------------------

int __fastcall TMainForm::AddFile (char *szFile, BOOL bSaveHeader)
{
	char	szBuf[200], *p;
	int		err;
    filetype	ft;

	try {	pai = new struct FileInfo;	}
	catch (...)
	{	ShowMessage (szErrorString[0]); return 1;		}
	memset (pai, 0, sizeof (struct FileInfo));

//	ShowMessage(szFile);
	ft = FileType (szFile, bSaveHeader);
	if (ft == ftUNK)
	{
		if (!bOptions[OPT_Silent])
        {
			wsprintf (szBuf, szErrorString[1], szErr, szFile);
			if (MessageBox(Handle, szBuf, "AVIcodec", MB_OKCANCEL | MB_ICONEXCLAMATION)
		    ==IDCANCEL)
            {
            	ChSilent->Checked = true;	// If cancel, we go in silent mode
	            bOptions[OPT_Silent] = true;
            }
//			ShowMessage (szBuf);
        }
		return 2;
	}

	err = 1;
    strcpy (szErr, szErrorString[2]);
	if (ft==ftAVI)		err = GetAVIinfo (szFile);
	else if (ft==ftMPG)	err = GetMPGinfo (szFile);
	else if (ft==ftOGG)	err = GetOGGinfo (szFile);
	else if (ft==ftRMF)	err = GetRMinfo (szFile);
	else if (ft==ftDSH)	err = GetDSHOWinfo (szFile);
	else if (ft==ftASF)	err = GetASFinfo (szFile);
	else if (ft==ftMP3)
    {
    	err = GetDSHOWinfo (szFile);
		pai->a[0].wAC = 0x55;
    }


	if (err && !bOptions[OPT_Silent])
	{
		wsprintf (szBuf, szErrorString[1], szErr, szFile);
		ShowMessage (szBuf);
	}
    FileInfoFill ();

	if ((pai->wVCsupp==ftUNS) || ((pai->a[0].wACsupp==ftUNS) && *pai->a[0].szACname))
		if (!ChFastScan->Checked)
        {
			// It not useful to do DSHOW test on header, and header files are something not readable by DShow
			if (!strstr (szFile, ".head") && !strstr (szFile, ".hdr"))
				DSHOWtest (szFile, false);	// test if DirectShow can handle sound & videco
        }
        else
        {
			if (pai->wVCsupp==ftUNS)	pai->wVCsupp=ftSNV;
			if (pai->a[0].wACsupp==ftUNS)	pai->a[0].wACsupp=ftSNV;
        }

	LV->Selected = NULL;				// To not have problems in LVChange ()
	TListItem *pLI = MainForm->LV->Items->Add();
	pLI->Data = pai;
/*	if ((pai->bFileAudio && (pai->a[0].wACsupp==ftSUP)) ||
	    (!pai->bFileAudio && (pai->wVCsupp==ftSUP) && (pai->a[0].wACsupp==ftSUP)))
    	pLI->ImageIndex = 1;*/
	if (pai->bFileAudio && (pai->a[0].wACsupp==ftSUP))
    	pLI->ImageIndex = 4;
	// Fichier video et dont le codec audio s'il existe est support
	else if ((!pai->bFileAudio && (pai->wVCsupp==ftSUP) && (!pai->bAStreamsNb || (pai->a[0].wACsupp==ftSUP))))
    	pLI->ImageIndex = 5;
	else if (!ChFastScan->Checked)
		pLI->ImageIndex = (pai->wVCsupp==ftSUP ? 3 : (pai->a[0].wACsupp==ftSUP) ? 2 : 0);
	else   	pLI->ImageIndex = -1;
	p = strrchr (szFile, '\\')+1;
	pLI->Caption = bOptions[OPT_HidePath] ? p : szFile;// En dernier, car ensuite appelle de LVChange
	ligIndex = pLI->Index;

	return 0;
}

// Explore a directory and possibly subdirectory
// -----------------------------------------------------------

void __fastcall TMainForm::ScanDir (char *szPath, BOOL recursive, BOOL bSaveHeader)
{
	struct	ffblk ffblk;
	int		done;
	char	szDir[MAXPATH],
			szExt[MAXPATH],
			*p;

	if (szPath[strlen (szPath)-1] != '\\')
		strcat (szPath, "\\");

	// First we scan files in current directory
    // We don't scan hidden files to let them hidden ...

	strcpy (szDir, szPath);
	strcat (szDir, "*.*");
	done = findfirst (szDir, &ffblk, FA_RDONLY | FA_ARCH);
	while (!done)
	{
		p = strrchr (ffblk.ff_name, '.');
        if (p && (strlen (p)>2))		// File with an extension
        {
			strcpy (szExt, p);
    	    strlwr (szExt);
//			szExt[3] = 0;				// I keep only .+the first 2 char.
        	if (strstr (szFilter, szExt))
	        {
				strcpy (szDir, szPath);
				strcat (szDir, ffblk.ff_name);
				AddFile (szDir, bSaveHeader);
			}
        }
		done = findnext(&ffblk);
	}

	if (!recursive)
		return;							// We stop

	// Second we scan directories in current directory to study them

	strcpy (szDir, szPath);
	strcat (szDir, "*.*");
	done = findfirst (szDir, &ffblk, FA_DIREC | FA_ARCH);
	while (!done)
	{
		if ((ffblk.ff_attrib & FA_DIREC) && (ffblk.ff_name[0] != '.'))
		{
			strcpy (szDir, szPath);
			strcat (szDir, ffblk.ff_name);
			strcat (szDir, "\\");
			ScanDir (szDir, recursive, bSaveHeader);
		}
		done = findnext(&ffblk);
	}
}

// Add information in columns 2, 3, 4, ...
// -----------------------------------------------------------

void __fastcall TMainForm::LVInsert(TObject *Sender, TListItem *Item)
{
	char	szBuf[100];
	int		i, c;

	for (i=1 ; i<NBCOL ; i++)
	{
		c = colId[i];
		if (c == -1)
		{
			Item->SubItems->Add ("");
			continue;
		}

		strcpy (szBuf, "");
        GetColData(c, szBuf);
		Item->SubItems->Add (szBuf);
	}
}

// Get information for columns col and put it in p
// -----------------------------------------------------------

void __fastcall TMainForm::GetColData(int col, char *p)
{
	int	min;

	switch (col)
	{
	case VFCC:
		strcpy (p, pai->szVC);
		break;
	case VCON:
		strcpy (p, pai->szVCname);
		break;
	case VCOS:
		strcpy (p, pai->wVCsupp==ftUNS ? szOtherString[1] :
           		pai->wVCsupp==ftSUP ? szOtherString[0] : szOtherString[20]);
		break;
	case VRES:
		wsprintf (p, "%ld * %ld", pai->dwWidth, pai->dwHeight);
		break;
	case VRAT:
		wsprintf (p, szOtherString[2], pai->dwVAvgBPS*8/1000);
		break;
	case VFRQ:
		if (pai->dwFps == (pai->dwFps /1000) * 1000)
				wsprintf (p, szOtherString[3], pai->dwFps/1000);
           else	wsprintf (p, szOtherString[4], pai->dwFps/1000, pai->dwFps - (int)(pai->dwFps/1000)*1000);
		break;
	case VTIM:
		min = pai->dwVSec / 60;
		wsprintf (p, "%d:%02d:%02d", pai->dwVSec/3600, min % 60, pai->dwVSec - 60 * min);
		break;
	case VARA:
		strcpy (p, pai->szAspRatio);
		break;
	case VSIZ:
		strcpy (p, szSize(pai->dwVSize));
		break;
	case ASTR:
		wsprintf (p, "%d", pai->bAStreamsNb);
		break;
	case ATSZ:
		strcpy (p, szSize(pai->dwATotSize));
		break;
	case A1COI:
		wsprintf (p, "0x%04X", pai->a[0].wAC);
		break;
	case A1CON:
		strcpy (p, pai->a[0].szACname);
		break;
	case A1COS:
		strcpy (p, pai->a[0].wACsupp==ftUNS ? szOtherString[1] :
           		pai->a[0].wACsupp==ftSUP ? szOtherString[0] : szOtherString[20]);
		break;
	case A1TIM:
		min = pai->a[0].dwASec / 60;
		wsprintf (p, "%d:%02d:%02d", pai->a[0].dwASec/3600, min % 60, pai->a[0].dwASec - 60 * min);
		break;
	case A1CHA:
		wsprintf (p, "%d", pai->a[0].bAChannels);
		break;
	case A1SNB:
		wsprintf (p, "%d", pai->a[0].dwASnb);
		break;
	case A1FRQ:
		wsprintf (p, "%d Hz", pai->a[0].dwASps);
		break;
	case A1RAT:
		wsprintf (p, szOtherString[2], pai->a[0].dwAAvgBPS*8/1000);
		break;
	case A1SIZ:
		strcpy (p, szSize(pai->a[0].dwASize));
		break;
	case A2COI:
		wsprintf (p, "0x%04X", pai->a[1].wAC);
		break;
	case A2CON:
		strcpy (p, pai->a[1].szACname);
		break;
	case A2COS:
		strcpy (p, pai->a[1].wACsupp==ftUNS ? szOtherString[1] :
           		pai->a[1].wACsupp==ftSUP ? szOtherString[0] : szOtherString[20]);
		break;
	case A2TIM:
		min = pai->a[1].dwASec / 60;
		wsprintf (p, "%d:%02d:%02d", pai->a[1].dwASec/3600, min % 60, pai->a[0].dwASec - 60 * min);
		break;
	case A2CHA:
		wsprintf (p, "%d", pai->a[1].bAChannels);
		break;
	case A2SNB:
		wsprintf (p, "%d", pai->a[1].dwASnb);
		break;
	case A2FRQ:
		wsprintf (p, "%d Hz", pai->a[1].dwASps);
		break;
	case A2RAT:
		wsprintf (p, szOtherString[2], pai->a[1].dwAAvgBPS*8/1000);
		break;
	case A2SIZ:
		strcpy (p, szSize(pai->a[1].dwASize));
		break;
	case FTYP:
		strcpy (p, pai->szFileType);
		break;
	case FRSZ:
		strcpy (p, szSize(pai->dwFileRealSizeKb, 1));
		break;
	case FTSZ:
		strcpy (p, szSize(pai->dwFileTheoSize));
		break;
	case FQUA:
		wsprintf (p, "%d %", pai->wFileQuality);
		break;
	case FDAT:
		strcpy (p, pai->szFileDate);
		break;
	}
}

// A line of the ListBox has been selected
//---------------------------------------------------------------------------

void __fastcall TMainForm::LVChange(TObject *Sender, TListItem *Item,
	  TItemChange Change)
{
	char	szBuf[200], szSupported[20];
	int		min, fps;

	if (Change != ctState)
    	return;
	if (Item->Selected)
	{
		pai = (struct FileInfo *)Item->Data;
		if (!pai)						// Happens some times ?!?
			return;

		min = pai->dwFileDuration / 60;
		wsprintf (szBuf, szOtherString[6], szSize(pai->dwFileRealSizeKb, 1), szSize(pai->dwFileTheoSize, 0, 10), pai->dwFileDuration/3600, min % 60, pai->dwFileDuration - 60 * min, pai->szFileType, pai->bAStreamsNb, pai->wFileQuality);
		FDetails->Text = szBuf;
		if (*pai->szFileUrl)
		{
			FWebBtn->Enabled = true;
			FWebBtn->Hint = pai->szFileUrl;
		}

		if (*pai->szVCname)
		{
			strcpy (szSupported, pai->wVCsupp==ftUNS ? szOtherString[1] :
            		pai->wVCsupp==ftSUP ? szOtherString[0] : szOtherString[20]);
            fps = pai->dwFps / 1000;
			wsprintf (szBuf, szOtherString[7], szSize(pai->dwVSize), pai->dwVAvgBPS*8/1000, fps, pai->dwFps - fps*1000, pai->dwWidth, pai->dwHeight, pai->szAspRatio, pai->szVC, pai->szVCname, szSupported);
			VDetails->Text = szBuf;
			if (*pai->szVCurl)
            {
		        VWebBtn->Enabled = true;
		        VWebBtn->Hint = pai->szVCurl;
            }
		}
        else	VDetails->Text = "";

		ADetails->Clear ();
		if (*pai->a[0].szACname)
		{
			for (int as=0 ; as<pai->bAStreamsNb ; as++)
            {
				strcpy (szSupported, pai->a[as].wACsupp==ftUNS ? szOtherString[1] :
    	        		pai->a[as].wACsupp==ftSUP ? szOtherString[0] : szOtherString[20]);
				wsprintf (szBuf, szOtherString[8], szSize(pai->a[as].dwASize), pai->a[as].dwAAvgBPS*8/1000, pai->a[as].dwASps, pai->a[as].bAChannels, pai->a[as].wAC, pai->a[as].szACname, szSupported);
				ADetails->AddItem (szBuf, NULL);
				if (!as)
					ADetails->Text = szBuf;
            }

			if (*pai->a[0].szACurl)
            {
		        AWebBtn->Enabled = true;
		        AWebBtn->Hint = pai->a[0].szACurl;
            }
		}
//        Item->Focused = true;
	}
	else
	{
		FDetails->Text = "";
		VDetails->Text = "";
		ADetails->Clear ();
		FWebBtn->Enabled = false;
        VWebBtn->Enabled = false;
        AWebBtn->Enabled = false;
	}
}
//---------------------------------------------------------------------------

int		SortedColumn;
bool	AscDescSort[NBCOL];

void __fastcall TMainForm::LVColumnClick(TObject *Sender, TListColumn *Column)
{
	SortedColumn = Column->Index;
	AscDescSort[SortedColumn] = !AscDescSort[SortedColumn];
	LV->AlphaSort();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::LVCompare(TObject *Sender, TListItem *Item1,
	  TListItem *Item2, int Data, int &Compare)
{
	switch (SortedColumn)
	{
	case 0 :
		Compare = Item1->Caption.AnsiCompare(Item2->Caption);
		break;
	default :
    	if (col[colId[SortedColumn]].cSortType == 'A')
				Compare = Item1->SubItems->Strings[SortedColumn-1].AnsiCompare(Item2->SubItems->Strings[SortedColumn-1]);
		else	Compare = atoi(Item1->SubItems->Strings[SortedColumn-1].c_str()) - atoi(Item2->SubItems->Strings[SortedColumn-1].c_str());
	}
	if (AscDescSort[SortedColumn]) Compare = -Compare;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::AboutBtnClick(TObject *Sender)
{
	char	szBuf[100];
    UINT 	i, j;

	for (i=j=0 ; i<=strlen(szMainForm[8]) ; i++)
		if (szMainForm[8][i] != '&')
	    	szBuf[j++] = szMainForm[8][i];
	szBuf[j]=0;
	AboutForm->Caption = szBuf;

	strcpy(szBuf, szMainForm[47]);
    strcat(szBuf, " :");
	AboutForm->LbVersion->Caption = szBuf;
	strcpy(szBuf, szMainForm[48]);
    strcat(szBuf, " :");
	AboutForm->LbEmail->Caption = szBuf;
	strcpy(szBuf, szMainForm[49]);
    strcat(szBuf, " :");
	AboutForm->LbWeb->Caption = szBuf;
	AboutForm->ShowModal ();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::CloseBtnClick(TObject *Sender)
{
	ClearList();
	Close();
}

//---------------------------------------------------------------------------

// Options button

void __fastcall TMainForm::OptionsBtnClick(TObject *Sender)
{
	int	n;

	OptionsForm->Left = MainForm->Left+45;
	OptionsForm->Top = MainForm->Top+120;
    OptionsForm->InitCombo();	// Because column settings may have changed
	OptionsForm->ShowModal ();

	// We clear the list & update it with the colums chosen in Options

	n = LV->Items->Count;
	if (n)
	{
        if (LV->Selected)
			ligIndex = LV->Selected->Index;
		LV->Selected = NULL;

		while (--n >= 0)
		{
			TListItem *p = LV->Items->Item[n];
			p->SubItems->Clear ();
			pai = (FileInfo *)p->Data;
			LVInsert(Sender, p);
		}
        if (ligIndex != -1)
			LV->Items->Item[ligIndex]->Selected = true;
	}
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::FormResize(TObject *Sender)
{
	if (ClientWidth < 632)	ClientWidth = 632;
	if (ClientHeight < 220)	ClientHeight = 220;
	PC->Width = ClientWidth;
	PC->Height = ClientHeight;
	LV->Width = ClientWidth - 24;
	LV->Height = ClientHeight - 162;

	FDetails->Width = ClientWidth - 64;
	FDetails->Top = ClientHeight - 94;
	FWebBtn->Left = ClientWidth - 52;
	FWebBtn->Top = ClientHeight - 93;

	VDetails->Width = ClientWidth - 64;
	VDetails->Top = ClientHeight - 74;
	VWebBtn->Left = ClientWidth - 52;
	VWebBtn->Top = ClientHeight - 73;

	ADetails->Width = ClientWidth - 64;
	ADetails->Top = ClientHeight - 54;
	AWebBtn->Left = ClientWidth - 52;
	AWebBtn->Top = ClientHeight - 53;

	LVvc->Width = ClientWidth - 24;
	LVvc->Height = ClientHeight - 68;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::FWebBtnClick(TObject *Sender)
{
	int result =(int)ShellExecute (0, "open", pai->szFileUrl, "", "", SW_SHOWNORMAL);
	if (result <= 32)
		if (!bOptions[OPT_Silent])
			ShowMessage (szErrorString[3]);
}

void __fastcall TMainForm::VWebBtnClick(TObject *Sender)
{
	int result =(int)ShellExecute (0, "open", pai->szVCurl, "", "", SW_SHOWNORMAL);
	if (result <= 32)
		if (!bOptions[OPT_Silent])
			ShowMessage (szErrorString[3]);
}

void __fastcall TMainForm::AWebBtnClick(TObject *Sender)
{
	int result =(int)ShellExecute (0, "open", pai->a[0].szACurl, "", "", SW_SHOWNORMAL);
	if (result <= 32)
		if (!bOptions[OPT_Silent])
			ShowMessage (szErrorString[3]);
}

// Export of all detail from Multimedia files
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExpFileListBtnClick(TObject *Sender)
{
	int		c, i, j, n;
	FILE	*fh;
    char	szBuf[1024];

	SaveDialog1->FileName = szMainForm[52];
	if (!SaveDialog1->Execute())
		return;

	fh = fopen (SaveDialog1->FileName.c_str (), "wt");
	if (fh == NULL)
    	return;

	if (bOptions[OPT_ExpCol])
	{
		strcpy(szBuf, "");
		for (i=0 ; i<NBCOL ; i++)
		{
			c = colId[i];
			if (c == -1)
				continue;
			strcat(szBuf, "\"");
			strcat(szBuf, col[c].szColName);
			strcat(szBuf, "\";");
        }
    }
    else
		strcpy(szBuf, szOtherString[9]);
	strcat(szBuf, "\n");
	fputs (szBuf, fh);

	n = LV->Items->Count;
	for (i=0 ; i<n ; i++)
    {
		TListItem *p = LV->Items->Item[i];
		pai = (struct FileInfo *)p->Data;

		if (bOptions[OPT_ExpCol])
        {
			strcpy(szBuf, p->Caption.c_str());
            strcat(szBuf, ";");
			for (j=1 ; j<NBCOL ; j++)
			{
				c = colId[j];
				if (c == -1)
					continue;
	            strcat(szBuf, "\"");
        		GetColData(c, szBuf+strlen(szBuf));
	            strcat(szBuf, "\";");
			}
            strcat(szBuf, "\n");
        }
        else
		 wsprintf (szBuf, "%s;%s;%s;%d;%d;%d;%d;%s;\"%s\";%s;%s;%d;\"%s\";%d;%d;%d;%d.%d;%d;%d;%d;%d;%04x;\"%s\";%s;%d;%d;%d;%d;%d;%d;%d;%04x;\"%s\";%s;%d;%d;%d;%d;%d;%d;%d\n",
        	p->Caption.c_str(),
			pai->szFileType,
			pai->szFileDate,
			pai->dwFileRealSize,
			pai->dwFileTheoSize,
			pai->dwFileDuration,
			pai->wFileQuality,
			pai->szVC,
			pai->szVCname,
			pai->szVCurl,
			pai->szVCdriver,
			pai->wVCsupp,
			pai->szAspRatio,
			pai->dwWidth,
			pai->dwHeight,
			pai->dwFramesNb,
			pai->dwFps/1000,
			pai->dwFps%1000,
			pai->dwVSec,
			pai->dwVSize,
			pai->dwVAvgBPS*8/1000,
			pai->bAStreamsNb,
			pai->a[0].wAC,
			pai->a[0].szACname,
			pai->a[0].szACurl,
			pai->a[0].wACsupp,
			pai->a[0].bAChannels,
			pai->a[0].dwASnb,
			pai->a[0].dwASps,
			pai->a[0].dwAAvgBPS*8/1000,
			pai->a[0].dwASec,
			pai->a[0].dwASize,
			pai->a[1].wAC,
			pai->a[1].szACname,
			pai->a[1].szACurl,
			pai->a[1].wACsupp,
			pai->a[1].bAChannels,
			pai->a[1].dwASnb,
			pai->a[1].dwASps,
			pai->a[1].dwAAvgBPS*8/1000,
			pai->a[1].dwASec,
			pai->a[1].dwASize
            );
		fputs (szBuf, fh);
    }
    fclose (fh);
}

// We accept drag & drop + Clipboard view in Main Window

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
	// Allow receiving draged files (for Drag & Drop)
	DragAcceptFiles(Handle, true);

	// Put AVIcodec in the list of clipboard viewer
    hWndNextViewer = SetClipboardViewer(Handle);
}

// Handling of dropped file

MESSAGE void __fastcall TMainForm::HandleDropFiles(TMessage& Msg)
{
	char	szFile[MAXPATH];
	HDROP hDrop = (HDROP)Msg.WParam;
    int	NbFiles = DragQueryFile (hDrop, -1, NULL, 0);

    for (int i=0 ; i<NbFiles ; i++)
    {
    	DragQueryFile (hDrop, i, szFile, MAXPATH-1);
		if (!AddFile (szFile, bOptions[OPT_SaveHeader]))
	        LV->Items->Item[0]->Selected = true;
	}
    DragFinish (hDrop);
}

// Handling of Clipboard content

MESSAGE void __fastcall TMainForm::OnDrawClipboard(TMessage& Msg)
{
	char	szBuf[255];

	if (!hWndNextViewer)	// Should not happened ?
    	return;
	TClipboard	*clip=Clipboard();
    if (clip==NULL)	// Some people have problems ("Unable to open clipboard")
    	return;
	if (clip->HasFormat(CF_TEXT))
    {
		clip->GetTextBuf(szBuf, 255);
		if (!strncmp(szBuf, "AVIcodec: ", 10) && (szMainForm[0] != NULL))
        {
			int	l = strlen(szBuf);
			clip->Clear();
            if (l != 10)
            {
				szBuf[l-1]='\0';	// To suppress quote around filename
				ExecCmdLine(szBuf+11);
            }
        }
    }
	clip->Close();
}

// Allow to select an object added via commandline the 1st time

MESSAGE void __fastcall TMainForm::OnSelect(TMessage& Msg)
{
	if (LV->Items->Count)
//    {
//		LV->Items->Item[0]->Selected = false;
//		LV->Items->Item[0]->Focused = true;
		LV->Items->Item[0]->Selected = true;
//        LV->ItemFocused = LV->Items->Item[0];
//    }
}

// If 'Scan recursively' is checked, then I check also 'Scan directory'

//---------------------------------------------------------------------------
// SECTION "PAGE VIDEO & PAGE AUDIO UPDATE
//---------------------------------------------------------------------------

SystemInfo *pSI;

void __fastcall TMainForm::LVvcInsert(TObject *Sender, TListItem *Item)
{
	Item->SubItems->Add (ci.szName);
	Item->SubItems->Add (ci.szVers);
	Item->SubItems->Add (ci.szComp);
	Item->SubItems->Add (ci.szFile);
}

//---------------------------------------------------------------------------
// Page change -> Codec page management
//---------------------------------------------------------------------------

int		vcSortedColumn;
bool	vcAscDescSort[NBCOL];

void __fastcall TMainForm::LVvcColumnClick(TObject *Sender, TListColumn *Column)
{
	vcSortedColumn = Column->Index;
	vcAscDescSort[vcSortedColumn] = !vcAscDescSort[vcSortedColumn];
	LVvc->AlphaSort();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::LVvcCompare(TObject *Sender, TListItem *Item1,
	  TListItem *Item2, int Data, int &Compare)
{
	switch (vcSortedColumn)
	{
	case 0 :
		Compare = Item1->Caption.AnsiCompare(Item2->Caption);
		break;
	default :
		Compare = Item1->SubItems->Strings[vcSortedColumn-1].AnsiCompare(Item2->SubItems->Strings[vcSortedColumn-1]);
	}
	if (vcAscDescSort[vcSortedColumn]) Compare = -Compare;
}

void __fastcall TMainForm::PCChange(TObject *Sender)
{
	if (PC->ActivePage == PageCodecs)
	{
		BOOL	bVideo = RBvideo->Checked;
		BOOL	bAudio = RBaudio->Checked;
		BOOL	bDshow = RBdsf->Checked;
		UINT	max;

		LVvc->Items->Clear ();			// Clear the list
		if (bOptions[OPT_DisableDMO])
        	RBdmo->Enabled = false;

		pSI = new SystemInfo;
		if (bVideo)
				max = pSI->getVCcount ();
        else if (bAudio)
        		max = pSI->getACcount ();
        else if (bDshow)
        		max = pSI->getDScount ();
		else	max = pSI->getDMOcount ();

		for (UINT i=0 ; i<max ; i++)
        {
			if (bVideo)
					pSI->getVC (i);
			else if (bAudio)
					pSI->getAC (i);
			else if (bDshow)
					pSI->getDS (i);
			else	pSI->getDMO (i);
			TListItem *p = MainForm->LVvc->Items->Add();
			p->Caption = ci.szId;
		}
		delete (pSI);

		if (!bVideo && !bAudio)
        {
			vcSortedColumn = 1;
			vcAscDescSort[vcSortedColumn] = 0;
			LVvc->AlphaSort();
        }
	}
}

// Export the content of ListBox
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExpVCListBtnClick(TObject *Sender)
{
	int		i, n;
	FILE	*fh;
    char	szBuf[1024];
    char	szFile[MAXPATH];

	if (RBvideo->Checked)
			strcpy (szFile, szMainForm[53]);
	else if (RBaudio->Checked)
			strcpy (szFile, szMainForm[54]);
	else if (RBdsf->Checked)
			strcpy (szFile, szMainForm[55]);
    else	strcpy (szFile, szMainForm[56]);

	SaveDialog1->FileName = szFile;
	if (!SaveDialog1->Execute())
		return;

	fh = fopen (SaveDialog1->FileName.c_str (), "wt");
	if (fh == NULL)
    	return;

	n = LVvc->Items->Count;
	for (i=0 ; i<n ; i++)
    {
		TListItem *p = LVvc->Items->Item[i];
		wsprintf (szBuf, "%s;%s;%s;%s;%s\n",
        	p->Caption.c_str(),
			p->SubItems->Strings[0].c_str(),
            p->SubItems->Strings[1].c_str(),
            p->SubItems->Strings[2].c_str(),
            p->SubItems->Strings[3].c_str());
		fputs (szBuf, fh);
    }
    fclose (fh);
}

// Load / Save parameters
//---------------------------------------------------------------------------

bool __fastcall TMainForm::LoadParam ()
{
	HKEY	hkey;
	DWORD	dwSize=256;
    TCHAR	data[256];
	char	szBuf[255];
    int	rc;
    bool	bInstalled;

	GetCurrentDirectory(MAXPATH-1, szInstallDir);
	strcpy (szInitDir, "");
	strcpy (szFilter, "");
	strcpy (szLang, "");
	colId[0] = FNAM;
	colId[1] = VCON;
	colId[2] = A1CON;
	for(int i=3 ; i<NBCOL ; i++)
		colId[i] = -1;

	rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\AVIcodec", 0, KEY_READ, &hkey);
	if (rc == ERROR_SUCCESS)
	{
		rc = RegQueryValueEx(hkey, "", NULL, NULL, data, &dwSize);
		if (!rc)	strcpy(szInstallDir, data);
		dwSize=256;
		rc = RegQueryValueEx(hkey, "szInitDir", NULL, NULL, data, &dwSize);
		if (!rc)	strcpy (szInitDir, data);
		dwSize=256;
		rc = RegQueryValueEx(hkey, "szFilter", NULL, NULL, data, &dwSize);
		if (!rc)	strcpy (szFilter, data);
        bInstalled = !rc;	// Because it is empty only the 1st time
		dwSize=256;
		rc = RegQueryValueEx(hkey, "szLang", NULL, NULL, data, &dwSize);
		if (!rc)	strcpy (szLang, data);

		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChScanRec", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_ScanRec] = *(DWORD *)data;
        ChScanRec->Checked = bOptions[OPT_ScanRec];
		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChSaveHeader", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_SaveHeader] = *(DWORD *)data;
        ChSaveHeader->Checked = bOptions[OPT_SaveHeader];
		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChClearList", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_ClearList] = *(DWORD *)data;
       	ChClearList->Checked = bOptions[OPT_ClearList];
		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChHidePath", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_HidePath] = *(DWORD *)data;
       	ChHidePath->Checked = bOptions[OPT_HidePath];
		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChSilent", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_Silent] = *(DWORD *)data;
		ChSilent->Checked = bOptions[OPT_Silent];
		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChFastScan", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_FastScan] = *(DWORD *)data;
        ChFastScan->Checked = bOptions[OPT_FastScan];
		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChDisableDMO", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_DisableDMO] = *(DWORD *)data;
		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChAssocFiles", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_AssocFiles] = *(DWORD *)data;
		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChAssocDir", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_AssocDir] = *(DWORD *)data;
		dwSize=256;
		rc = RegQueryValueEx(hkey, "ChExpCol", NULL, NULL, data, &dwSize);
		if (!rc)	bOptions[OPT_ExpCol] = *(DWORD *)data;

		for (int i=0 ; i<NBCOL ; i++)
    	{
			dwSize=256;
			wsprintf(szBuf, "colId%d", i);
			rc = RegQueryValueEx(hkey, szBuf, NULL, NULL, data, &dwSize);
			if (!rc)	colId[i] = *(DWORD *)data;

			// Sauvegarde de la dimension des colonnes

			dwSize=256;
			wsprintf(szBuf, "colWidth%d", i);
			rc = RegQueryValueEx(hkey, szBuf, NULL, NULL, data, &dwSize);
			if (!rc && colId[i])
            	LV->Columns->Items[i]->Width = *(DWORD *)data;
		}

		dwSize=256;
		rc = RegQueryValueEx(hkey, "WindowLeft", NULL, NULL, data, &dwSize);
		if (!rc)	MainForm->Left = *(DWORD *)data;
		dwSize=256;
		rc = RegQueryValueEx(hkey, "WindowTop", NULL, NULL, data, &dwSize);
		if (!rc)	MainForm->Top = *(DWORD *)data;
		dwSize=256;
		rc = RegQueryValueEx(hkey, "WindowWidth", NULL, NULL, data, &dwSize);
		if (!rc)	MainForm->Width = *(DWORD *)data;
		dwSize=256;
		rc = RegQueryValueEx(hkey, "WindowHeight", NULL, NULL, data, &dwSize);
		if (!rc)	MainForm->Height = *(DWORD *)data;
        rc = ERROR_SUCCESS;
	}
	RegCloseKey(hkey);
	strlwr (szFilter);
    return bInstalled;
}

void __fastcall TMainForm::SaveParam ()
{
	HKEY	hkey;
    TCHAR	data[256];
	int 	rc = 0;
	char	szBuf[255];

	rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\AVIcodec", 0, KEY_SET_VALUE, &hkey);
	if (rc == ERROR_SUCCESS)
    {
		rc = RegSetValueEx(hkey, "szInitDir", NULL, REG_SZ, szInitDir, lstrlen(szInitDir)+1);
		rc = RegSetValueEx(hkey, "szFilter", NULL, REG_SZ, szFilter, lstrlen(szFilter)+1);
		rc = RegSetValueEx(hkey, "szLang", NULL, REG_SZ, szLang, lstrlen(szLang)+1);

		*(DWORD *)data = (DWORD)bOptions[OPT_ScanRec];
		rc = RegSetValueEx(hkey, "ChScanRec", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)bOptions[OPT_SaveHeader];
		rc = RegSetValueEx(hkey, "ChSaveHeader", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)bOptions[OPT_ClearList];
		rc = RegSetValueEx(hkey, "ChClearList", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)bOptions[OPT_Silent];
		rc = RegSetValueEx(hkey, "ChSilent", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)bOptions[OPT_HidePath];
		rc = RegSetValueEx(hkey, "ChHidePath", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)bOptions[OPT_DisableDMO];
		rc = RegSetValueEx(hkey, "ChDisableDMO", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)bOptions[OPT_FastScan];
		rc = RegSetValueEx(hkey, "ChFastScan", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)bOptions[OPT_AssocFiles];
		rc = RegSetValueEx(hkey, "ChAssocFiles", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)bOptions[OPT_AssocDir];
		rc = RegSetValueEx(hkey, "ChAssocDir", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)bOptions[OPT_ExpCol];
		rc = RegSetValueEx(hkey, "ChExpCol", NULL, REG_DWORD, data, sizeof(DWORD));

		for (int i=0 ; i<NBCOL ; i++)
    	{
			wsprintf(szBuf, "colId%d", i);
			*(DWORD *)data = (DWORD)colId[i];
			rc = RegSetValueEx(hkey, szBuf, NULL, REG_DWORD, data, sizeof(DWORD));

			wsprintf(szBuf, "colWidth%d", i);
			*(DWORD *)data = (DWORD)LV->Columns->Items[i]->Width;
			rc = RegSetValueEx(hkey, szBuf, NULL, REG_DWORD, data, sizeof(DWORD));
		}

		*(DWORD *)data = (DWORD)MainForm->Left;
		rc = RegSetValueEx(hkey, "WindowLeft", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)MainForm->Top;
		rc = RegSetValueEx(hkey, "WindowTop", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)MainForm->Width;
		rc = RegSetValueEx(hkey, "WindowWidth", NULL, REG_DWORD, data, sizeof(DWORD));
		*(DWORD *)data = (DWORD)MainForm->Height;
		rc = RegSetValueEx(hkey, "WindowHeight", NULL, REG_DWORD, data, sizeof(DWORD));

		rc = RegCloseKey(hkey);
	}
}

// Load language texts
//---------------------------------------------------------------------------

#define MAXITBUF    500

char *szErrorString[30];				// Error strings
char *szOtherString[30];				// Other strings
char *szMainForm[60];					// Main strings
char *szOptionsForm[20];				// OptionsForm
extern struct ASPECTRATIO ara[MAXARA];

int __fastcall TMainForm::InitText ()
{
    char    szBuf[MAXITBUF],
    		szFile[MAXPATH], *p;
    int		table, idx, len;
	FILE	*fh;

	wsprintf (szFile, "%s\\lang\\%s.txt", szInstallDir, szLang);
	fh = fopen (szFile, "rt");
	if (fh == NULL)	// If I don't find the language file, I try english one
	{
		wsprintf (szFile, "%s\\lang\\English.txt", szInstallDir);
		fh = fopen (szFile, "rt");
		if (fh == NULL)
		{
    		strcpy (szErr, "Error on opening language file");
        	return -1;
        }
    }

	table = 0;
	while (fgets (szBuf, MAXITBUF, fh))
    {
		if ((*szBuf==';') || (*szBuf=='\n'))  // Comment or newline
            continue;
		len = strlen (szBuf);
		szBuf[len-1] = 0;	// We delete last character (\n)
        if (!strcmp (szBuf, "<Column name>"))
        {	table = 1; idx = 0;	continue; }
        if (!strcmp (szBuf, "<Error strings>"))
        {	table = 2; idx = 0;	continue; }
        if (!strcmp (szBuf, "<Other strings>"))
        {	table = 3; idx = 0;	continue; }
        if (!strcmp (szBuf, "<MainForm>"))
        {	table = 4; idx = 0;	continue; }
        if (!strcmp (szBuf, "<OptionsForm>"))
        {	table = 5; idx = 0;	continue; }

		// Storage in corresponding table
		if (!table)	continue;
		p = new char [len];
		strcpy (p, szBuf);
		switch (table)
        {
		case 1: col[idx++].szColName = p; break;
		case 2:	szErrorString[idx++] = p; break;
		case 3: szOtherString[idx++] = p; break;
		case 4: szMainForm[idx++] = p; break;
		case 5: szOptionsForm[idx++] = p; break;
        }
    }
	fclose (fh);

	strcpy (ara[3].szAra, szOtherString[11]);

	PageAVI->Caption = szMainForm[0];
	PageAVI->Hint = szMainForm[1];
	PageCodecs->Caption = szMainForm[2];
	PageCodecs->Hint = szMainForm[3];
	SelFileBtn->Caption = szMainForm[4];
	SelFileBtn->Hint = szMainForm[5];
	OpenDialog1->Title = szMainForm[5];
	OptionsBtn->Caption = szMainForm[6];
	OptionsBtn->Hint = szMainForm[7];
	AboutBtn->Caption = szMainForm[8];
	AboutBtn->Hint = szMainForm[9];
	ExpFileListBtn->Caption = szMainForm[10];
	ExpFileListBtn->Hint = szMainForm[11];
	CloseBtn->Caption = szMainForm[12];
	CloseBtn->Hint = szMainForm[13];
//	ChScanDir->Caption = szMainForm[14];
//	ChScanDir->Hint = szMainForm[15];
	ChScanRec->Caption = szMainForm[16];
	ChScanRec->Hint = szMainForm[17];
	ChSaveHeader->Caption = szMainForm[18];
	ChSaveHeader->Hint = szMainForm[19];
	ChClearList->Caption = szMainForm[20];
	ChClearList->Hint = szMainForm[21];
//	ChSilent->Caption = szMainForm[22];
//	ChSilent->Hint = szMainForm[23];
	ChSilent->Caption = szOptionsForm[5];
	ChSilent->Hint = szOptionsForm[6];
	ChHidePath->Caption = szMainForm[24];
	ChHidePath->Hint = szMainForm[25];
	FDetails->Text = szMainForm[26];
	VDetails->Text = szMainForm[27];
	ADetails->Text = szMainForm[28];
	RBvideo->Caption = szMainForm[29];
	RBvideo->Hint = szMainForm[30];
	RBaudio->Caption = szMainForm[31];
	RBaudio->Hint = szMainForm[32];
	RBdsf->Caption = szMainForm[33];
	RBdsf->Hint = szMainForm[34];
	ExpVCListBtn->Caption = szMainForm[10];
	ExpVCListBtn->Hint = szMainForm[11];
	for (int i=0 ; i<5 ; i++)
	    LVvc->Columns->Items[i]->Caption = szMainForm[35+i];
	SelDirBtn->Caption = szMainForm[40];
	SelDirBtn->Hint = szMainForm[41];
	FDetails->Hint = szMainForm[42];
	VDetails->Hint = szMainForm[43];
	ADetails->Hint = szMainForm[44];
	FWebBtn->Caption = szMainForm[49];
	VWebBtn->Caption = szMainForm[49];
	AWebBtn->Caption = szMainForm[49];
	strcpy(szBuf, szMainForm[49]);

	RBdmo->Caption = szMainForm[45];
	RBdmo->Hint = szMainForm[46];
	ChFastScan->Caption = szMainForm[50];
	ChFastScan->Hint = szMainForm[51];

    return 0;
}
//---------------------------------------------------------------------------

char * __fastcall TMainForm::szSize (DWORD size, int unit, int pos)
{
	static char szTmp[50];
    DWORD	val=size, reste=0;

//	while (val > 2048)
	while (val > 1024)
	{	reste = val & 0x03FF; val >>= 10; unit++;	}
	if (val < 20)
			wsprintf (szTmp+pos, "%d.%d %s", val, reste/10, !unit ? szOtherString[16] : unit==1 ? szOtherString[17] : unit==2 ? szOtherString[18] : szOtherString[19]);
	else	wsprintf (szTmp+pos, "%d %s", val, !unit ? szOtherString[16] : unit==1 ? szOtherString[17] : unit==2 ? szOtherString[18] : szOtherString[19]);
	return szTmp+pos;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ChClearListClick(TObject *Sender)
{
	bOptions[OPT_ClearList] = ChClearList->Checked;
}
void __fastcall TMainForm::ChScanRecClick(TObject *Sender)
{
	bOptions[OPT_ScanRec] = ChScanRec->Checked;
}
void __fastcall TMainForm::ChHidePathClick(TObject *Sender)
{
	bOptions[OPT_HidePath] = ChHidePath->Checked;
}
void __fastcall TMainForm::ChSaveHeaderClick(TObject *Sender)
{
	bOptions[OPT_SaveHeader] = ChSaveHeader->Checked;
}
void __fastcall TMainForm::ChFastScanClick(TObject *Sender)
{
	bOptions[OPT_FastScan] = ChFastScan->Checked;
}

//---------------------------------------------------------------------------
// Store data in clipboard

void __fastcall TMainForm::LVKeyDown(TObject *Sender, WORD &Key, TShiftState Shift)
{
	if (!LV->Selected)	return;
	ligIndex = LV->Selected->Index;
	TListItem *p = LV->Items->Item[ligIndex];

	if (Key==VK_DELETE)
    {
		delete p->Data;
		p->Delete();
		if (LV->Items->Count)
        {
			if (LV->Items->Count <= ligIndex)
            	ligIndex = LV->Items->Count-1;
	        LV->Items->Item[ligIndex]->Selected = true;
        }
    }

    if (!Shift.Contains(ssCtrl))	return;

	pai = (struct FileInfo *)p->Data;
	if (!pai)	return;

	TClipboard	*clip=Clipboard();	// Instance to windows Clipboard
    if (Key==(WORD)'F')				// Ctrl-F
        clip->SetTextBuf(p->Caption.c_str());
    else if (Key==(WORD)'V')		// Ctrl-V
        clip->SetTextBuf(pai->szVCname);
    else if (Key==(WORD)'A')		// Ctrl-A
        clip->SetTextBuf(pai->a[0].szACname);
	clip->Close();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
	// Retire from clipboard viewer list
	if (hWndNextViewer)
		ChangeClipboardChain(Handle, hWndNextViewer);
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::LVDblClick(TObject *Sender)
{
	if (!LV->Selected)	return;

	TListItem *p = LV->Items->Item[LV->Selected->Index];
	pai = (struct FileInfo *)p->Data;

	int result =(int)ShellExecute(0, "open", p->Caption.c_str(), "", "", SW_SHOWNORMAL);
	if (result <= 32)
		if (!bOptions[OPT_Silent])
			ShowMessage (szErrorString[3]);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ChSilentClick(TObject *Sender)
{
	bOptions[OPT_Silent] = ChSilent->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::LVColumnRightClick(TObject *Sender,
      TListColumn *Column, TPoint &Point)
{
	if (MessageBox(Handle, "Do you want to remove this column ?", "Column setting", MB_YESNO | MB_ICONQUESTION)
		    ==IDYES)
    {
		for (int i=Column->Index ; i<NBCOL-1 ; i++)
        	colId[i] = colId[i+1];
       	colId[NBCOL-1] = -1;
        InitColumns();
    }
}
//---------------------------------------------------------------------------

