//---------------------------------------------------------------------------
// Mpeg decoding is in a separate file due to its bigger complexity
//---------------------------------------------------------------------------
// Inspired from bbdmux (bbtool19_src.zip) by Brent Beyeler, beyeler@home.com
// Audio header analyse from FlaskMpeg by Alberto Vigata, ultraflask@yahoo.com
// Audio doc : http://www.dv.co.yu/mpgscript/mpeghdr.htm
// Trailers :
// - ftp://ftp.csst.net/csst/Dolby  (login:csst, passwd:csst)
// - http://www.digital-digest.com/dvd/downloads/trailers.html
// - http://www.cinenow.com/fr/vobtrailer.php3
// -----------------------------------------------------------
//      { MPEG_NORM_FILM,   352, 240, 1 },
//      { MPEG_NORM_PAL,    352, 288, 3 },
//      { MPEG_NORM_NTSC,   352, 240, 4 },
//      { MPEG_NORM_PAL_S,  480, 576, 3 },
//      { MPEG_NORM_NTSC_S, 480, 480, 4 },
//      { MPEG_NORM_OTHER, }

#include <vcl.h>						// ShowMesssage
#pragma hdrstop

#include <io.h>							// filelength
#include <stdio.h>						// fopen

#include "Main.h"						// MainForm			
#include "MpegInfo.h"
#include "fileinfo.h"
#include "multinfo.h"

extern struct FileInfo *pai;

//---------------------------------------------------------------------------

#pragma package(smart_init)

// Class Init / Close
// -----------------------------------------------------------

int __fastcall MpegInfo::Init (LPCSTR szFile)
{
	fh = fopen (szFile, "rb");
	if (fh == NULL)
	{	strcpy (szErr, szErrorString[7]);	return -1;	}

	try {	bfr = new char [BUFFER_SIZE];	}
	catch (...)
	{	ShowMessage (szErrorString[0]); return 1;		}

	bytecount = 0;
	bufcount = 0;
    for (int i=0 ; i<8 ; i++)
		masks[i] = 1<<i;
	if (!refill_buffer ())
	{	strcpy (szErr, szErrorString[14]);	return -2;	}

    return 0;
}

void __fastcall MpegInfo::Close ()
{
	delete bfr;
	fclose (fh);
}

// Fill buffer, return true if ok, false if end of file

bool __fastcall MpegInfo::refill_buffer ()
{
	byteidx = 0;
	bitidx = 8;
	bytecount += bufcount;
	bufcount = fread (bfr, 1, BUFFER_SIZE, fh);
//	eobs = (bufcount <= 0);
//  Lets try to fill the buffer just once
	if (!bytecount)	eobs = (bufcount <= 0);	else	eobs = true;
	return !eobs;
}

// -----------------------------------------------------------
// MPG general functions, simplied (no buffer refill)
// -----------------------------------------------------------

// Read 1 bit from the bit stream
// -----------------------------------------------------------

unsigned int __fastcall MpegInfo::get1bit ()
{
	unsigned int bit;

	if (eobs)
		return 0;

	bit = (bfr[byteidx] & masks[bitidx - 1]) >> (bitidx - 1);
	bitidx--;
	if (!bitidx)
	{
		bitidx = 8;
		byteidx++;
		if (byteidx == bufcount)
			refill_buffer ();
	}
	return bit;
}

// Read N bit from the bit stream
// -----------------------------------------------------------

unsigned int __fastcall MpegInfo::getbits (int N)
{
	DWORD	val = 0;
	int		i = N;

	if (eobs)
		return 0;

	// Optimization in case we are on byte boundary and want to read multiple of bytes!
	if ((bitidx == 8) && !(N & 7))
	{
		i = N >> 3;
		while (i > 0)
		{
			val = (val << 8) | (DWORD) bfr[byteidx];
			byteidx++;
			if (byteidx == bufcount)
				if (!refill_buffer ())
                	break;;
			i--;
		}
    }
	else while (i > 0)
	{
		val = (val << 1) | (DWORD) get1bit ();
		i--;
    }

	return val;
}

// Look ahead for the next N bits from the bit stream
// -----------------------------------------------------------

unsigned int __fastcall MpegInfo::look_ahead (int N)
{
	DWORD val = 0;
	unsigned char *tmp_bfr = bfr;
	unsigned char tmp_bfr1[4];
	int j = N;
	int eo_bs = eobs;
	int buf_count = bufcount;
	int bit_idx = bitidx;
	int byte_idx = byteidx;

	while (j > 0)
	{
		if (eo_bs)
			return 0;
		val <<= 1;
		val |= ((DWORD)tmp_bfr[byte_idx] & masks[bit_idx - 1]) >> (bit_idx - 1);
		bit_idx--;
		j--;
		if (!bit_idx)
		{
			bit_idx = 8;
			byte_idx++;
			if (byte_idx == buf_count)
			{
				if (buf_count == BUFFER_SIZE)
				{
					int	len = fread (tmp_bfr1, 1, 4, fh);
					fseek (fh, -len, SEEK_CUR);
					if (len != 4)
							eo_bs = true;
					else	tmp_bfr = &tmp_bfr1[0];
				}
				else
					eo_bs = true;
				byte_idx = 0;
			}
		}
	}
	return val;
}

// -----------------------------------------------------------
// MPG info research
// -----------------------------------------------------------

#define MAXAUDIOMPEGSTREAM			10	// False, just to have a number

int __fastcall MpegInfo::ParseFile ()
{
	char	szTrack[10];
	bool	bFirstPSC = true,
    		bAudioStream[MAXAUDIOMPEGSTREAM],
    		bAC3Stream[MAXAUDIOMPEGSTREAM],
			bDTS,
			bLPCM,
            bEnd,
			bMpeg2,
            bMpeg1audio;
	int		as,
//    		ar_vcd[16] = {	0, 100, 67, 70, 76, 81, 84, 89, 94, 98, 103, 107, 112, 116, 120, 0   },
    		ar_svcd[16] = { 0, 133, 178, 221, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    		fr[9] = { 0,
				23976,		// 1 (23.976 fps) - FILM
				24000,		// 2 (24.000 fps)
				25000,		// 3 (25.000 fps) - PAL
				29970,		// 4 (29.970 fps) - NTSC
				30000,		// 5 (30.000 fps)
				50000,		// 6 (50.000 fps) - PAL noninterlaced
				59940,		// 7 (59.940 fps) - NTSC noninterlaced
				60000,		// 8 (60.000 fps)
            },
			mpeg1_bitrate_table[3][15] = {
	{ 0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448 },
	{ 0, 32, 48, 56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320, 384 },
	{ 0, 32, 40, 48,  56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320 }},
			mpeg2_bitrate_table[3][15] = {
	{ 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, },
	{ 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, },
	{ 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, }};
    int    AC3Acmod[]={ 2, 1, 2, 3, 3, 4, 4, 5 };
	int    AC3SampleFreq[4]={48000,44100,32000,0};
	struct
	{
		WORD	bit_rate;
		WORD	frm_size[3];
	} AC3Framesize[] = {
      { 32  ,{64   ,69   ,96   } },
      { 32  ,{64   ,70   ,96   } },
      { 40  ,{80   ,87   ,120  } },
      { 40  ,{80   ,88   ,120  } },
      { 48  ,{96   ,104  ,144  } },
      { 48  ,{96   ,105  ,144  } },
      { 56  ,{112  ,121  ,168  } },
      { 56  ,{112  ,122  ,168  } },
      { 64  ,{128  ,139  ,192  } },
      { 64  ,{128  ,140  ,192  } },
      { 80  ,{160  ,174  ,240  } },
      { 80  ,{160  ,175  ,240  } },
      { 96  ,{192  ,208  ,288  } },
      { 96  ,{192  ,209  ,288  } },
      { 112 ,{224  ,243  ,336  } },
      { 112 ,{224  ,244  ,336  } },
      { 128 ,{256  ,278  ,384  } },
      { 128 ,{256  ,279  ,384  } },
      { 160 ,{320  ,348  ,480  } },
      { 160 ,{320  ,349  ,480  } },
      { 192 ,{384  ,417  ,576  } },
      { 192 ,{384  ,418  ,576  } },
      { 224 ,{448  ,487  ,672  } },
      { 224 ,{448  ,488  ,672  } },
      { 256 ,{512  ,557  ,768  } },
      { 256 ,{512  ,558  ,768  } },
      { 320 ,{640  ,696  ,960  } },
      { 320 ,{640  ,697  ,960  } },
      { 384 ,{768  ,835  ,1152 } },
      { 384 ,{768  ,836  ,1152 } },
      { 448 ,{896  ,975  ,1344 } },
      { 448 ,{896  ,976  ,1344 } },
      { 512 ,{1024 ,1114 ,1536 } },
      { 512 ,{1024 ,1115 ,1536 } },
      { 576 ,{1152 ,1253 ,1728 } },
      { 576 ,{1152 ,1254 ,1728 } },
      { 640 ,{1280 ,1393 ,1920 } },
      { 640 ,{1280 ,1394 ,1920 } }};
	WORD	code;
    unsigned char	stream_id;
    int		aspect_ratio,
    		n,
            idx,
    		a_type,
            a_val,
			InitTime;
    DWORD	bitrate;
	DWORD	PES_header_data_length;
	WORD	sync;

	for (n=0 ; n<MAXAUDIOMPEGSTREAM ; n++)
    	bAudioStream[n] = bAC3Stream[n] = false;

	bLPCM = bDTS = bEnd = false;
	InitTime = -1;
	code = getbits (16);				// To force first getbits (16);
	do
    {
		while (code && !eobs)			// We go further till obtaining 0x0000
        	code = (code << 8) | getbits (8);
		while (!look_ahead (8) && !eobs)
        	getbits (8);
       	code = getbits (16);
		if (eobs && !bEnd && bufcount)	// We have seen the header, now lets jump to the end of file
        {
            bEnd = true;
			fseek (fh, -BUFFER_SIZE, SEEK_END);
            refill_buffer ();
			eobs = false;
            continue;
        }
       	if ((code>>8) != 0x01)			// Packet Start Code Prefix
			continue;

		stream_id = code & 0x00FF;
        switch (code)
        {
		case MPEG_PACK_START_CODE:
        	if (bFirstPSC)
            {
				bMpeg2 = (look_ahead (2) == 1);
				strcpy (pai->szVC, !bMpeg2 ? "MPG1" : "MPG2");
				pai->dwFileHeadSize = byteidx;
                bFirstPSC = false;
            }
            break;

		case MPEG_PROGRAM_END_CODE:
			eobs = true;
            break;

		case MPEG_SEQUENCE_CODE:		// Inside video stream
			if (pai->dwWidth)			// Already studied
				break;

			if (!*pai->szVC)			// Happens when no MPEG_PACK_START_CODE (XingCD)
				strcpy (pai->szVC, "MPG1");

        	pai->dwWidth = getbits (12);
        	pai->dwHeight = getbits (12);
			aspect_ratio = getbits (4);
/*            if (!bMpeg2)
					pai->wAspRatio = ar_vcd[aspect_ratio];	// Uncorrect, but why ?
            else */
            pai->wAspRatio = ar_svcd[aspect_ratio];

			idx = getbits (4);
			pai->dwFps = fr[idx];
			idx = getbits (18);
            if (idx != 0x3FFFF) 		// 0x3FFFF if VBR Mpeg file
	            bitrate = 400 * idx;	// 1152 for VCD, 2500 for SVCD, 3800 for DVD
			pai->dwVAvgBPS = bitrate / 8;
			pai->wVCsupp = ftSUP;		// It is supported with correct player
			getbits (6);				// To be byte aligned
            break;

		case MPEG_GOP_CODE:				// Group-of-Pictures
			int drop_flag;
			unsigned	hour,
            			minute,
                        second;
//                        frame;
			drop_flag = getbits (1);
/*            if (drop_flag)				// Data is not usable ? (DTS pb)
            {							//
				getbits (7);
            	break;
            }*/
			hour = getbits (5);
			minute = getbits (6);
			getbits (1);
			second = getbits (6);
//			frame = getbits (6);
			getbits (2);
			getbits (3);				// To be byte aligned

			if (InitTime == -1)			// 
				InitTime = second + 60 * minute + 3600 * hour;
        	if (bEnd)
				pai->dwFileDuration = pai->a[0].dwASec = pai->dwVSec = (second + 60 * minute + 3600 * hour) - InitTime;
 			break;

		case MPEG_PRIVATE_STREAM_1:
			// Explanations on AC3 from ac3dec by Aaron Holtzman (included in FlaskMepeg source)
			DWORD	PES_packet_length;	// Data
			BYTE	substream_id;

			PES_packet_length = getbits (16);
			getbits (16);
			PES_header_data_length = getbits (8);
			for (n=0; n<(int)PES_header_data_length; n++)
				getbits (8);
			substream_id = getbits (8);

			if ((substream_id >= SUBSTREAM_AC3_0)  && (substream_id <= SUBSTREAM_AC3_8))
            {
				if (bAC3Stream[substream_id - SUBSTREAM_AC3_0] == true)	// Already studied
        	        break;
				bAC3Stream[substream_id - SUBSTREAM_AC3_0] = true;
				pai->bAStreamsNb++;
    	        as = pai->bAStreamsNb-1;
				if (as >= MAXAUDIOSTREAMS)
                	break;

				n = PES_packet_length - PES_header_data_length - 10;
				sync = getbits (8);
				do
        	    {
					sync = (sync << 8) + getbits (8);
					if (sync == AUDIO_AC3_SYNC_WORD)
                    	break;
                	n--;
	            }	while (n);
    	        if (!n)						// AC3 sync not found
	            	break;

				pai->a[as].wAC = 0x2000;			// AC3
				strcpy (pai->a[as].szACname, searchACname (pai->a[as].wAC));
				wsprintf (szTrack, " [0x%x]", substream_id);
                strcat (pai->a[as].szACname, szTrack);
				getbits (16);

				a_type = getbits (2);		// If 3, error
				pai->a[as].dwASps = AC3SampleFreq[a_type];

				a_val = getbits (6);
				pai->a[as].dwAAvgBPS = 1000 / 8 * AC3Framesize[a_val].bit_rate;
//				frame_size = AC3Framesize[a_val].frm_size[a_type];
//				bsi->bsid = getbits (5);	// AC-3 version number
//				bsi->bsmod = getbits (3);	// Audio service provided by the stream
  				getbits (8);
  				a_type = getbits (3);		// Audio coding mode
  				pai->a[as].bAChannels = AC3Acmod[a_type];	// Number of full bandwidth channels
				pai->a[as].wACsupp = ftSUP;	// It is supported with correct player

  				pai->dwFileHeadSize = byteidx;
  				getbits (5);				// To be byte aligned
            }
			else if ((substream_id >= SUBSTREAM_DTS_0)  && (substream_id <= SUBSTREAM_DTS_8))
            {
				if (!bDTS)
                {
					if (!bOptions[OPT_Silent])
						ShowMessage (szErrorString[15]);
                    bDTS = true;

					pai->bAStreamsNb++;
    		        as = pai->bAStreamsNb-1;
					if (as >= MAXAUDIOSTREAMS)
            	    	break;
					pai->a[as].wAC = 0x0008;			// DTS
                }
            }
			else if ((substream_id >= SUBSTREAM_PCM_0)  && (substream_id <= SUBSTREAM_PCM_F))
            {
				if (!bLPCM)
                {
					if (!bOptions[OPT_Silent])
						ShowMessage (szErrorString[16]);
                    bLPCM = true;
                }
            }
 			break;

		case MPEG_AUDIO_C0_CODE:
		case MPEG_AUDIO_C1_CODE:
		case MPEG_AUDIO_C2_CODE:
			if (bAudioStream[stream_id - 0xc0] == true)	// Already studied
                break;
			bAudioStream[stream_id - 0xc0] = true;
            as = pai->bAStreamsNb;
			if (as >= MAXAUDIOSTREAMS)
               	break;

			// I don't try to understand PES data ...

			PES_packet_length = getbits (16);
			if (!bMpeg2)
            {
				PES_header_data_length = 7;
				while (look_ahead(8) == 0xFF)
                {
    	            getbits(8);
					PES_header_data_length++;
                }
   	            getbits(16);
   	            getbits(32);
   	            getbits(8);
			}
            else
            {
				getbits (16);
				PES_header_data_length = getbits (8);
				for (n=0; n<(int)PES_header_data_length; n++)
					getbits (8);
				substream_id = getbits (8);
            }

			n = PES_packet_length - PES_header_data_length - 10;
			while ((look_ahead (12) != AUDIO_MPEG_SYNC_WORD) && n--)
            	getbits (8);
			if (!n)
            	break;
           	getbits (12);

			bMpeg1audio = getbits (1);		// version, 1=Mpeg1, 0=Mpeg2
			a_type = 4-getbits (2);		// layer (3 = layer I, 2 = layer II, 1 = layer III)
			pai->a[as].wAC = (a_type==1 ? 0x0050 : a_type==2 ? 0x0051 : a_type==3 ? 0x0053 : 0);
			strcpy (pai->a[as].szACname, searchACname (pai->a[as].wAC));
			wsprintf (szTrack, " [0x%x]", stream_id);
			strcat (pai->a[as].szACname, szTrack);

			getbits (1);				// error protection (0 = enabled)
            a_val = getbits (4);		// bitrate_index
            if (a_val == 0xF)	break;	// Invalid value
			pai->a[as].dwAAvgBPS = 1000 / 8 * (bMpeg1audio ? mpeg1_bitrate_table : mpeg2_bitrate_table)[a_type-1][a_val];

			a_type = getbits (2);		// sampling_freq
			pai->a[as].dwASps = (a_type==0 ? 44100 : a_type==1 ? 48000 : a_type==2 ? 32000 : 0);
			if (!bMpeg1audio)
				pai->a[as].dwASps >>= 1;		// Divide by 2

            getbits (2);				// padding + extension
			if (getbits (2) == 3)		// mode (0=stereo, 1=joint stereo, 2=dual channel, 3=mono)
					pai->a[as].bAChannels = 1;
			else	pai->a[as].bAChannels = 2;
            getbits (6);				// To be byte aligned : mode_ext + copyright + original + emphasis
			pai->a[as].wACsupp = ftSUP;	// It is supported with correct player

			// Valid audio stream
			pai->bAStreamsNb++;
        }
    }	while (!eobs);
    return 0;
}

