// Header file for MultInfo.cpp

#include <dir.h>						// MAXPATH
#include <vfw.h>						// HACMDRIVERID
#include <vector.h>						// vector

#include "fileinfo.h"					// MAXSZERR
#include "dmoreg.h"						// IEnumDMO

#define MAXCODID	40					// GUID = 36+1
#define MAXCODNAME	50
#define	MAXCODVERS	20
#define	MAXCODCOMP	50
#define MAXCODFILE	MAXPATH
#define MAXCODURL	100

// The 3 main aspect ratio existing
// -----------------------------------------------------------

struct ASPECTRATIO
{
	char	szAra[8];
	int		num100, min100, max100;
};
#define MAXARA	4
extern struct ASPECTRATIO ara[MAXARA];

char *searchAspRatio (int aratio);

// Video & Audio codec description
// -----------------------------------------------------------

struct VIDEOCODEC
{
	char szFCC[5];
	char szName[MAXCODNAME];
	char szURL[MAXCODURL];
};

struct AUDIOCODEC
{
	WORD	code;
	char	szName[MAXCODNAME];
	char	szURL[MAXCODURL];
};

extern vector<struct VIDEOCODEC>	vcd;
extern vector<struct AUDIOCODEC>	acd;
int InitCodecs (char *szInstallDir);

// Functions

int searchvc (char *szFCC);
char *searchVCname (char *szFCC);
char *searchVCurl (char *szFCC);

int searchac (int code);
char *searchACname (int code);
char *searchACurl (int code);

// "Codecs & filters" information

struct CodecInfo
{
	char	szId[MAXCODID],					// Four CC, Format Tag Index, GUID
			szName[MAXCODNAME],
			szVers[MAXCODVERS],
			szComp[MAXCODCOMP],
			szFile[MAXCODFILE];
};

// Not inside SuystemInfo because of access problem from callback function
extern struct CodecInfo ci;
extern vector<CodecInfo>	sys_vc;		// System Video Codec/Filter
extern vector<CodecInfo>	sys_ac;		// System Audio Codec/Filter
extern vector<CodecInfo>	sys_ds;   	// System DirectShow Filter
extern vector<CodecInfo>	sys_dmo;   	// System DirectX Media Object (DMO)
extern BOOL bCIinit;					// Is CodecInfo initialized ?

class SystemInfo
{
private:

	int __fastcall readVersion ();
	void __fastcall InitCodecInfo ();
	int __fastcall DXnext (IEnumMoniker *pEnumCat);
	int __fastcall DMOnext (IEnumDMO *pEnumCat);
	static BOOL CALLBACK acmDriverEnumCallback (HACMDRIVERID hadid, DWORD dwInstance, DWORD fdwSupport);

public:
	__fastcall SystemInfo();
	char * __fastcall getErrorText(HRESULT hr);

	int	__fastcall getVCcount ()	{	return sys_vc.size();	};
	int	__fastcall getACcount ()	{	return sys_ac.size();	};
	int	__fastcall getDScount ()	{	return sys_ds.size();	};
	int	__fastcall getDMOcount ()	{	return sys_dmo.size();	};
	int	__fastcall getVC (UINT idx)
    {	if (idx<sys_vc.size())	{	ci = sys_vc[idx]; return 0;	} 	else return -1;	};
	int	__fastcall getAC (UINT idx)
    {	if (idx<sys_ac.size())	{	ci = sys_ac[idx]; return 0;	} 	else return -1;	};
	int	__fastcall getDS (UINT idx)
    {	if (idx<sys_ds.size())	{	ci = sys_ds[idx]; return 0;	} 	else return -1;	};
	int	__fastcall getDMO (UINT idx)
    {	if (idx<sys_dmo.size())	{	ci = sys_dmo[idx]; return 0;	} 	else return -1;	};
};

